# This file is generated by running "yarn install" inside your project.
# Manual changes might be lost - proceed with caution!

{ lib, nodejs, stdenv, fetchurl, writeText, git, cacert, writeShellApplication }:
{ src, overrideAttrs ? null, ... } @ args:

let

  yarnPath = ./.yarn/releases/yarn-3.1.1.cjs;
  lockfile = ./yarn.lock;
  cacheFolder = ".yarn/cache";

  # Call overrideAttrs on a derivation if a function is provided.
  optionalOverride = fn: drv:
    if fn == null then drv else drv.overrideAttrs fn;

  # Common attributes between Yarn derivations.
  drvCommon = {
    # Make sure the build uses the right Node.js version everywhere.
    buildInputs = [ nodejs ];
    # Tell node-gyp to use the provided Node.js headers for native code builds.
    npm_config_nodedir = nodejs;
    # Tell node-pre-gyp to never fetch binaries / always build from source.
    npm_config_build_from_source = "true";
    # Defines the shell alias to run Yarn.
    postHook = ''
      yarn() {
        CI=1 node "${yarnPath}" "$@"
      }
    '';
  };

  # Create derivations for fetching dependencies.
  cacheDrvs = let
    builder = builtins.toFile "builder.sh" ''
      source $stdenv/setup
      cd "$src"
      HOME="$TMP" yarn_cache_folder="$TMP" CI=1 \
        node '${yarnPath}' nixify fetch-one $locator
      # Because we change the cache dir, Yarn may generate a different name.
      mv "$TMP/$(sed 's/-[^-]*\.[^-]*$//' <<< "$outputFilename")"-* $out
    '';
  in lib.mapAttrs (locator: { filename, sha512 }: stdenv.mkDerivation {
    inherit src builder locator;
    name = lib.strings.sanitizeDerivationName locator;
    buildInputs = [ nodejs git cacert ];
    outputFilename = filename;
    outputHashMode = "flat";
    outputHashAlgo = "sha512";
    outputHash = sha512;
  }) cacheEntries;

  # Create a shell snippet to copy dependencies from a list of derivations.
  mkCacheBuilderForDrvs = drvs:
    writeText "collect-cache.sh" (lib.concatMapStrings (drv: ''
      cp ${drv} '${drv.outputFilename}'
    '') drvs);


  # Main project derivation.
  project = stdenv.mkDerivation (drvCommon // {
    inherit src;
    name = "q2tg";
    # Disable Nixify plugin to save on some unnecessary processing.
    yarn_enable_nixify = "false";

    configurePhase = ''
      # Copy over the Yarn cache.
      rm -fr '${cacheFolder}'
      mkdir -p '${cacheFolder}'
      pushd '${cacheFolder}' > /dev/null
      source ${mkCacheBuilderForDrvs (lib.attrValues cacheDrvs)}
      popd > /dev/null

      # Yarn may need a writable home directory.
      export yarn_global_folder="$TMP"

      # Some node-gyp calls may call out to npm, which could fail due to an
      # read-only home dir.
      export HOME="$TMP"

      # running preConfigure after the cache is populated allows for
      # preConfigure to contain substituteInPlace for dependencies as well as the
      # main project. This is necessary for native bindings that maybe have
      # hardcoded values.
      runHook preConfigure



      # Run normal Yarn install to complete dependency installation.
      yarn install --immutable --immutable-cache

      runHook postConfigure
    '';

    buildPhase = ''
      runHook preBuild
      runHook postBuild
    '';

    installPhase = ''
      runHook preInstall

      mkdir -p "$out/libexec/$name" "$out/bin"

      # Move the package contents to the output directory.
      # - If the package.json has a `files` field, only files matching those patterns are copied
      # - Otherwise all files are copied
      yarn pack --out package.tgz
      tar xzvf package.tgz --directory "$out/libexec/$name" --strip-components=1

      cp .yarnrc* "$out/libexec/$name"
      cp --recursive .yarn "$out/libexec/$name"

      # If the project uses the node-modules linker, then
      # include the node_modules folder in the package.
      if [ -d node_modules ]; then
        cp --recursive node_modules "$out/libexec/$name"
      fi

      cd "$out/libexec/$name"

      # Invoke a plugin internal command to setup binaries.
      yarn nixify install-bin $out/bin

      # A package with node_modules doesn't need the cache
      if [ -d node_modules ]; then
        yarn cache clean
      fi

      runHook postInstall
    '';

    passthru = {
      inherit nodejs;
      yarn = writeShellApplication {
        name = "yarn";
        runtimeInputs = [ nodejs ];
        text = ''
          ${yarnPath} --cwd ${project}/libexec/${project.name} "$@"
        '';
      };
    };
  });

cacheEntries = {
"@types/date-and-time@npm:0.13.0" = { filename = "@types-date-and-time-npm-0.13.0-b9a5b45537-a430aaaa77.zip"; sha512 = "a430aaaa77479746b026295ee6dab19497cc838f7a575569e3a2304841396e1aeba431bc528a86ffce037c12e371f0f4847085f6c99117f8c5809a97560f2971"; };
"@types/lodash@npm:4.14.182" = { filename = "@types-lodash-npm-4.14.182-1073aac722-7dd137aa9d.zip"; sha512 = "7dd137aa9dbabd632408bd37009d984655164fa1ecc3f2b6eb94afe35bf0a5852cbab6183148d883e9c73a958b7fec9a9bcf7c8e45d41195add6a18c34958209"; };
"@types/node@npm:18.6.3" = { filename = "@types-node-npm-18.6.3-29cdfaa5cd-38495b8fd2.zip"; sha512 = "38495b8fd27200d2b7ab9ccd8c1e2475d2411fba15330f2cba869a85ef79dd42382a2658c8dd298ace8c21ed3bfafcdea408faecd74928b144377d07d00e7c8c"; };
"@types/prompts@npm:2.4.0" = { filename = "@types-prompts-npm-2.4.0-5b5caba459-ec75655f07.zip"; sha512 = "ec75655f07fbc3cb2df60a3e3bdd13bfda0554a16f5541285c4b1cc0610756838a2863dccfc911cbb1385492054c07210512164e3dbef272ec638d631783b278"; };
"date-and-time@npm:2.4.1" = { filename = "date-and-time-npm-2.4.1-d71b5ac2d2-593d216604.zip"; sha512 = "593d216604deb833ef5a0677b84d6bc0a2cb94aab9f68423200e296b22f07c835a0d4f4472c897622143c1bab22d89ace616c8c529dea3387265f4c662dd6f01"; };
"dotenv@npm:16.0.1" = { filename = "dotenv-npm-16.0.1-d99e72bb95-f459ffce07.zip"; sha512 = "f459ffce07b977b7f15d8cc4ee69cdff77d4dd8c5dc8c85d2d485ee84655352c2415f9dd09d42b5b5985ced3be186130871b34e2f3e2569ebc72fbc2e8096792"; };
"eviltransform@npm:0.2.2" = { filename = "eviltransform-npm-0.2.2-91128ce3b2-db4efeea52.zip"; sha512 = "db4efeea529f4ab173eff7184ebd2273c2b64e4a5f772e5d3d250be773c46e60f3a176d7f7ced9ebb95cae9d8aee195d3aca2215bd4305453ffa0f74c48aa846"; };
"lodash@npm:4.17.21" = { filename = "lodash-npm-4.17.21-6382451519-eb835a2e51.zip"; sha512 = "eb835a2e51d381e561e508ce932ea50a8e5a68f4ebdd771ea240d3048244a8d13658acbd502cd4829768c56f2e16bdd4340b9ea141297d472517b83868e677f7"; };
"nodejs-base64@npm:2.0.0" = { filename = "nodejs-base64-npm-2.0.0-6bc5adeae2-1e9fc06396.zip"; sha512 = "1e9fc06396bd77caba14f93eec40751f00c4570346ebbaf9ded746f8309d0adc8a18bf62c55a9e0288854df7eb61ee9942b316548fd1e185de59213ce4f0fbde"; };
"tsc@npm:2.0.4" = { filename = "tsc-npm-2.0.4-e331b066cb-50b1024088.zip"; sha512 = "50b10240887424c66454687f0fca4e319713b35cb44f0dab228719eb4dd757251f83c3d79e60d29dfbe90c2d6f1272f793b83e75a9fca2c622b4213ad3eb8cb1"; };
"typescript@npm:4.7.4" = { filename = "typescript-npm-4.7.4-65aa6ffb42-5750181b1c.zip"; sha512 = "5750181b1cd7e6482c4195825547e70f944114fb47e58e4aa7553e62f11b3f3173766aef9c281783edfd881f7b8299cf35e3ca8caebe73d8464528c907a164df"; };
"@prisma/client@npm:4.6.0" = { filename = "@prisma-client-npm-4.6.0-fe27a80ce5-9ea358059c.zip"; sha512 = "9ea358059c01d5ed18b683a1bf4c713c08eb084622aadafa0b80d0439d5ea46e8930efa3fc72a915914f306f505bc7874954466ba13fdd385ca39e8009702e03"; };
"@types/cli-progress@npm:3.11.0" = { filename = "@types-cli-progress-npm-3.11.0-889a906352-d440162233.zip"; sha512 = "d4401622333e888925b47c5d5bb0b89dddae17cc020f909a64ad7275b326bf3c6e9cd467f625a197fd958a1e49220d32f4a2b0bf2948fee330c719a9b985674e"; };
"@types/fluent-ffmpeg@npm:2.1.20" = { filename = "@types-fluent-ffmpeg-npm-2.1.20-46c4294bd5-f810a12a42.zip"; sha512 = "f810a12a427f8e6568cec2ef250693e533112b66b7cd9ca9e10fe32b2aecb85623b8bb4ae5e51e47f79e27cdb969d3115720588b9f6fc7e4d17b4d4dc5e0d677"; };
"@types/sharp@npm:0.31.0" = { filename = "@types-sharp-npm-0.31.0-a646a0f79c-c7698de181.zip"; sha512 = "c7698de18183a0574bf2f703226a5a7dc32b8a7340b52790eff0f6775b8ad708b6782f48fcaaf4d1be15fd76215c81c3ea55595d55484bf7ce870923c8174294"; };
"cli-progress@npm:3.11.2" = { filename = "cli-progress-npm-3.11.2-6cecb7043d-147d26b80c.zip"; sha512 = "147d26b80ceaa24d72f0354d1b58b7f3567b928bf5943be879de31cf16b0a4f1d059984e2e35a664d7d27ae3e7fafd69fd94b35f462c8879caf96d7f31eac442"; };
"tmp-promise@npm:3.0.3" = { filename = "tmp-promise-npm-3.0.3-6df4ad8df9-f854f5307d.zip"; sha512 = "f854f5307dcee6455927ec3da9398f139897faf715c5c6dcee6d9471ae85136983ea06662eba2edf2533bdcb0fca66d16648e79e14381e30c7fb20be9c1aa62c"; };
"axios@npm:0.27.2" = { filename = "axios-npm-0.27.2-dbe3a48aea-38cb754046.zip"; sha512 = "38cb7540465fe8c4102850c4368053c21683af85c5fdf0ea619f9628abbcb59415d1e22ebc8a6390d2bbc9b58a9806c874f139767389c862ec9b772235f06854"; };
"fluent-ffmpeg@npm:2.1.2" = { filename = "fluent-ffmpeg-npm-2.1.2-692c218f68-ab7ed90948.zip"; sha512 = "ab7ed909486298f33b991af051c38e40410ae5b03b0b6d33d0855636a0b56330ffe9efed6c7eedb00f1c6c713c795cbdb283f6f2216db925ae5e737a22c6a97f"; };
"prompts@npm:2.4.2" = { filename = "prompts-npm-2.4.2-f5d25d5eea-d8fd1fe638.zip"; sha512 = "d8fd1fe63820be2412c13bfc5d0a01909acc1f0367e32396962e737cb2fc52d004f3302475d5ce7d18a1e8a79985f93ff04ee03007d091029c3f9104bffc007d"; };
"file-type@npm:17.1.4" = { filename = "file-type-npm-17.1.4-72b08f1f60-07ae79cda0.zip"; sha512 = "07ae79cda01a38c2dedec85acfa7d5532c8334d4bc8542e0ab97e27d1bc496d325ea245bb546394f43ac19673fad6ef50fc38a9d70462ae2f5a28ab8721d1037"; };
"silk-sdk@npm:0.2.2" = { filename = "silk-sdk-npm-0.2.2-2c0636bebf-e6314428f1.zip"; sha512 = "e6314428f1e406e59ffb400bb0493dfa056d1ffcf36afa60de59cfe22937ce70f5b7a08757510880755dbf1cb9137a298947641b10e7d1369bdd421674afdbec"; };
"@prisma/engines-version@npm:4.6.0-53.2e719efb80b56a3f32d18a62489de95bb9c130e3" = { filename = "@prisma-engines-version-npm-4.6.0-53.2e719efb80b56a3f32d18a62489de95bb9c130e3-7777b7153b-88510cec37.zip"; sha512 = "88510cec375f1505953121a41fa464f26be4022859cfe40659e4cb15c7e8d22d163203a30d65f2e66c828287cf029eb92ab6ba7325c6b3f551f92fb52424477b"; };
"@types/node@npm:17.0.21" = { filename = "@types-node-npm-17.0.21-7d68eb6a13-89dcd2fe82.zip"; sha512 = "89dcd2fe82f21d3634266f8384e9c865cf8af49685639fbdbd799bdd1040480fb1e8eeda2d3b9fce41edbe704d2a4be9f427118c4ae872e8d9bb7cbeb3c41a94"; };
"log4js@npm:6.6.1" = { filename = "log4js-npm-6.6.1-00e7698906-68f29e7b8b.zip"; sha512 = "68f29e7b8b0efa2f987e65ee122fde0bc806dd7ad20f0a9f2da316696423e42381ba348bd6e2f6a2feba042a6ed33c14955210741f47ab897b3204aba8bcd84b"; };
"oicq@https://0w.al/AhKp.tar.gz" = { filename = "oicq-https-7250a03304-1ac9013190.zip"; sha512 = "1ac901319005191fa3974f35efedca40fda12589e564d0f38065314eccf761f08adc6004d623d25b29edb45418e327790f8e21c343fadc45312c0b3be0ee0284"; };
"follow-redirects@npm:1.15.1" = { filename = "follow-redirects-npm-1.15.1-6b191885cd-6aa4e3e3cd.zip"; sha512 = "6aa4e3e3cdfa3b9314801a1cd192ba756a53479d9d8cca65bf4db3a3e8834e62139245cd2f9566147c8dfe2efff1700d3e6aefd103de4004a7b99985e71dd533"; };
"async@npm:3.2.3" = { filename = "async-npm-3.2.3-e9d6b79c88-c4bee57ab2.zip"; sha512 = "c4bee57ab2249af3dc83ca3ef9acfa8e822c0d5e5aa41bae3eaf7f673648343cd64ecd7d26091ffd357f3f044428b17b5f00098494b6cf8b6b3e9681f0636ca1"; };
"kleur@npm:3.0.3" = { filename = "kleur-npm-3.0.3-f6f53649a4-df82cd1e17.zip"; sha512 = "df82cd1e172f957bae9c536286265a5cdbd5eeca487cb0a3b2a7b41ef959fc61f8e7c0e9aeea9c114ccf2c166b6a8dd45a46fd619c1c569d210ecd2765ad5169"; };
"sisteransi@npm:1.0.5" = { filename = "sisteransi-npm-1.0.5-af60cc0cfa-aba6438f46.zip"; sha512 = "aba6438f46d2bfcef94cf112c835ab395172c75f67453fe05c340c770d3c402363018ae1ab4172a1026a90c47eaccf3af7b6ff6fa749a680c2929bd7fa2b37a4"; };
"tmp@npm:0.2.1" = { filename = "tmp-npm-0.2.1-a9c8d9c0ca-8b12146541.zip"; sha512 = "8b1214654182575124498c87ca986ac53dc76ff36e8f0e0b67139a8d221eaecfdec108c0e6ec54d76f49f1f72ab9325500b246f562b926f85bcdfca8bf35df9e"; };
"which@npm:1.3.1" = { filename = "which-npm-1.3.1-f0ebb8bdd8-f2e185c624.zip"; sha512 = "f2e185c6242244b8426c9df1510e86629192d93c1a986a7d2a591f2c24869e7ffd03d6dac07ca863b2e4c06f59a4cc9916c585b72ee9fa1aa609d0124df15e04"; };
"cac@npm:6.7.12" = { filename = "cac-npm-6.7.12-2abc45f504-c0d4129eb3.zip"; sha512 = "c0d4129eb30fc43449e9078ac37bb3b837aab6261236a6642a6fb9d839bb6a41e191e1f2776f87569535db07dcbf4937680419126215b4c17c9dba4351d1bd5e"; };
"string-width@npm:4.2.3" = { filename = "string-width-npm-4.2.3-2c27177bae-e52c10dc3f.zip"; sha512 = "e52c10dc3fbfcd6c3a15f159f54a90024241d0f149cf8aed2982a2d801d2e64df0bf1dc351cf8e95c3319323f9f220c16e740b06faecd53e2462df1d2b5443fb"; };
"readable-web-to-node-stream@npm:3.0.2" = { filename = "readable-web-to-node-stream-npm-3.0.2-682f5de297-8c56cc62c6.zip"; sha512 = "8c56cc62c68513425ddfa721954875b382768f83fa20e6b31e365ee00cbe7a3d6296f66f7f1107b16cd3416d33aa9f1680475376400d62a081a88f81f0ea7f9c"; };
"bindings@npm:1.5.0" = { filename = "bindings-npm-1.5.0-77ce1d213c-65b6b48095.zip"; sha512 = "65b6b48095717c2e6105a021a7da4ea435aa8d3d3cd085cb9e85bcb6e5773cf318c4745c3f7c504412855940b585bdf9b918236612a1c7a7942491de176f1ae7"; };
"form-data@npm:4.0.0" = { filename = "form-data-npm-4.0.0-916facec2d-01135bf867.zip"; sha512 = "01135bf8675f9d5c61ff18e2e2932f719ca4de964e3be90ef4c36aacfc7b9cb2fceb5eca0b7e0190e3383fe51c5b37f4cb80b62ca06a99aaabfcfd6ac7c9328c"; };
"strtok3@npm:7.0.0-alpha.9" = { filename = "strtok3-npm-7.0.0-alpha.9-7e615744e3-30639f4f57.zip"; sha512 = "30639f4f57685617eb3f7c6345249f2795d2d6ba820965fc0bc314eb08183931c6beb38dbbbbadb233eda0499f090e80894f4e356b95492e611d7d0ff4f29ea5"; };
"token-types@npm:5.0.0-alpha.2" = { filename = "token-types-npm-5.0.0-alpha.2-1fdc0995e7-ee23eeed6f.zip"; sha512 = "ee23eeed6f383b1072d99781d62fc7840f1296a96d47e636e36fca757debd7eb4274d31fcd2d56997606eede00b12b1e61a64610fe0ed7807d6b1c4dcf5ccc6b"; };
"date-format@npm:4.0.13" = { filename = "date-format-npm-4.0.13-64e384acc1-b4584fdd43.zip"; sha512 = "b4584fdd43d31e9c900da775cce4806903ba207e21f79d99959858141849d41ec919f941d1fb2e4efd2355ce1909ca1fe41a5184568c74cb540cf24c4f8bb39b"; };
"flatted@npm:3.2.6" = { filename = "flatted-npm-3.2.6-a309e17eb3-33b87aa88d.zip"; sha512 = "33b87aa88dfa40ca6ee31d7df61712bbbad3d3c05c132c23e59b9b61d34631b337a18ff2b8dc5553acdc871ec72b741e485f78969cf006124a3f57174de29a0e"; };
"rfdc@npm:1.3.0" = { filename = "rfdc-npm-1.3.0-272f288ad8-fb2ba8512e.zip"; sha512 = "fb2ba8512e43519983b4c61bd3fa77c0f410eff6bae68b08614437bc3f35f91362215f7b4a73cbda6f67330b5746ce07db5dd9850ad3edc91271ad6deea0df32"; };
"long@npm:4.0.0" = { filename = "long-npm-4.0.0-ecd96a31ed-16afbe8f74.zip"; sha512 = "16afbe8f749c7c849db1f4de4e2e6a31ac6e617cead3bdc4f9605cb703cd20e1e9fc1a7baba674ffcca57d660a6e5b53a9e236d7b25a295d3855cca79cc06744"; };
"pngjs@npm:6.0.0" = { filename = "pngjs-npm-6.0.0-31d8d85681-ab6c285086.zip"; sha512 = "ab6c285086060087097eab9fe6b5a528a24f9e79c03dea2b4fd6264ed4fdb5beff4a3257eeeaf2a9dc18249b539609c2a4e4013c567164a1f6b5ba2c974d5ecb"; };
"sharp@npm:0.31.3" = { filename = "sharp-npm-0.31.3-7533cf1669-29fd1dfbc6.zip"; sha512 = "29fd1dfbc616c6389f53f366cec342b4353d9f2a37e98952ca273db38dca57dfa0f336322d6d763f0fae876042ead22fd86ffe26d70c32ade2458d421db60d04"; };
"debug@npm:4.3.4" = { filename = "debug-npm-4.3.4-4513954577-3dbad3f94e.zip"; sha512 = "3dbad3f94ea64f34431a9cbf0bafb61853eda57bff2880036153438f50fb5a84f27683ba0d8e5426bf41a8c6ff03879488120cf5b3a761e77953169c0600a708"; };
"typescript@patch:typescript@npm%3A4.7.4#~builtin<compat/typescript>::version=4.7.4&hash=493e53" = { filename = "typescript-patch-56cded6317-96d3030cb0.zip"; sha512 = "96d3030cb01143570567cb4f3a616b10df65f658f0e74e853e77a089a6a954e35c800be7db8b9bfe9a1ae05d9c2897e281359f65e4caa1caf266368e1c4febd3"; };
"isexe@npm:2.0.0" = { filename = "isexe-npm-2.0.0-b58870bd2e-26bf6c5480.zip"; sha512 = "26bf6c5480dda5161c820c5b5c751ae1e766c587b1f951ea3fcfc973bafb7831ae5b54a31a69bd670220e42e99ec154475025a468eae58ea262f813fdc8d1c62"; };
"rimraf@npm:3.0.2" = { filename = "rimraf-npm-3.0.2-2cb7dac69a-87f4164e39.zip"; sha512 = "87f4164e396f0171b0a3386cc1877a817f572148ee13a7e113b238e48e8a9f2f31d009a92ec38a591ff1567d9662c6b67fd8818a2dbbaed74bc26a87a2a4a9a0"; };
"emoji-regex@npm:8.0.0" = { filename = "emoji-regex-npm-8.0.0-213764015c-d4c5c39d5a.zip"; sha512 = "d4c5c39d5a9868b5fa152f00cada8a936868fd3367f33f71be515ecee4c803132d11b31a6222b2571b1e5f7e13890156a94880345594d0ce7e3c9895f560f192"; };
"is-fullwidth-code-point@npm:3.0.0" = { filename = "is-fullwidth-code-point-npm-3.0.0-1ecf4ebee5-44a30c2945.zip"; sha512 = "44a30c29457c7fb8f00297bce733f0a64cd22eca270f83e58c105e0d015e45c019491a4ab2faef91ab51d4738c670daff901c799f6a700e27f7314029e99e348"; };
"file-uri-to-path@npm:1.0.0" = { filename = "file-uri-to-path-npm-1.0.0-1043ac6206-b648580bdd.zip"; sha512 = "b648580bdd893a008c92c7ecc96c3ee57a5e7b6c4c18a9a09b44fb5d36d79146f8e442578bc0e173dc027adf3987e254ba1dfd6e3ec998b7c282873010502144"; };
"streamroller@npm:3.1.2" = { filename = "streamroller-npm-3.1.2-56c1517978-4215bf57fe.zip"; sha512 = "4215bf57fe80c583ba200075f9a62b9be9a3d7491201905bf7b5a09d7e87903a47f7c62b1ce0fd2161dbd497cac41068e8f48dda0f9bfceb084487de5798f043"; };
"axios@npm:1.2.1" = { filename = "axios-npm-1.2.1-4056ddcb4c-c4dc4e1190.zip"; sha512 = "c4dc4e119064c9aed09a3de309bedb797a139a6fb372223aafe3e0c10a7d4a14e4d3e9c9d309467fadb9d2b490b891ee3df96ef5b55716bb971910466ff9f0c5"; };
"probe-image-size@npm:7.2.3" = { filename = "probe-image-size-npm-7.2.3-2b6ee36e6f-1a5eeb8f5c.zip"; sha512 = "1a5eeb8f5cb979172144a5d7a017c70fcd664ccc8af9ad3a803903ee81864abea4036adae4fc6e66e9ae21bd3ce0febefaf1f32e65a77ff226b2eb61e9e4978c"; };
"asynckit@npm:0.4.0" = { filename = "asynckit-npm-0.4.0-c718858525-7b78c451df.zip"; sha512 = "7b78c451df768adba04e2d02e63e2d0bf3b07adcd6e42b4cf665cb7ce899bedd344c69a1dcbce355b5f972d597b25aaa1c1742b52cffd9caccb22f348114f6be"; };
"strip-ansi@npm:6.0.1" = { filename = "strip-ansi-npm-6.0.1-caddc7cb40-f3cd25890a.zip"; sha512 = "f3cd25890aef3ba6e1a74e20896c21a46f482e93df4a06567cebf2b57edabb15133f1f94e57434e0a958d61186087b1008e89c94875d019910a213181a14fc8c"; };
"@tokenizer/token@npm:0.3.0" = { filename = "@tokenizer-token-npm-0.3.0-4441352cc5-1d575d02d2.zip"; sha512 = "1d575d02d2a9f0c5a4ca5180635ebd2ad59e0f18b42a65f3d04844148b49b3db35cf00b6012a1af2d59c2ab3caca59451c5689f747ba8667ee586ad717ee58e1"; };
"peek-readable@npm:5.0.0-alpha.5" = { filename = "peek-readable-npm-5.0.0-alpha.5-290fcea102-cab949ed45.zip"; sha512 = "cab949ed457dac95ae191dd412c6a0ba05e8db4842fd51704ccf2c8c16d6f3ceeefc997e8caea584a0395f229e468c0203a38a8d0ec68cfef8bacc157a006dcb"; };
"ieee754@npm:1.2.1" = { filename = "ieee754-npm-1.2.1-fb63b3caeb-5144c0c981.zip"; sha512 = "5144c0c9815e54ada181d80a0b810221a253562422e7c6c3a60b1901154184f49326ec239d618c416c1c5945a2e197107aee8d986a3dd836b53dffefd99b5e7e"; };
"detect-libc@npm:2.0.1" = { filename = "detect-libc-npm-2.0.1-2699cb2ac4-ccb05fcabb.zip"; sha512 = "ccb05fcabbb555beb544d48080179c18523a343face9ee4e1a86605a8715b4169f94d663c21a03c310ac824592f2ba9a5270218819bb411ad7be578a527593d7"; };
"ts-node@npm:10.9.1" = { filename = "ts-node-npm-10.9.1-6c268be7f4-090adff130.zip"; sha512 = "090adff1302ab20bd3486e6b4799e90f97726ed39e02b39e566f8ab674fd5bd5f727f43615debbfc580d33c6d9d1c6b1b3ce7d8e3cca3e20530a145ffa232c35"; };
"combined-stream@npm:1.0.8" = { filename = "combined-stream-npm-1.0.8-dc14d4a63a-49fa4aeb49.zip"; sha512 = "49fa4aeb4916567e33ea81d088f6584749fc90c7abec76fd516bf1c5aa5c79f3584b5ba3de6b86d26ddd64bae5329c4c7479343250cfe71c75bb366eae53bb7c"; };
"mime-types@npm:2.1.35" = { filename = "mime-types-npm-2.1.35-dd9ea9f3e2-89a5b7f1de.zip"; sha512 = "89a5b7f1def9f3af5dad6496c5ed50191ae4331cc5389d7c521c8ad28d5fdad2d06fd81baf38fed813dc4e46bb55c8145bb0ff406330818c9cf712fb2e9b3836"; };
"ms@npm:2.1.2" = { filename = "ms-npm-2.1.2-ec0c1512ff-673cdb2c31.zip"; sha512 = "673cdb2c3133eb050c745908d8ce632ed2c02d85640e2edb3ace856a2266a813b30c613569bf3354fdf4ea7d1a1494add3bfa95e2713baa27d0c2c71fc44f58f"; };
"log4js@npm:6.4.1" = { filename = "log4js-npm-6.4.1-b00dd7826e-0614949662.zip"; sha512 = "0614949662314573ec7dcd841769a4d23d8cb8268685458a40fcd94f2ae6ec628234cfb9a6bc17821fb6ea6ce3765e779b4966ba1cf918f393dc37155a3615cb"; };
"node-addon-api@npm:5.0.0" = { filename = "node-addon-api-npm-5.0.0-c0def7fecf-7c5e2043ac.zip"; sha512 = "7c5e2043ac37f6108784d94ed73a44ae6d3e68eb968de60680922fc6bc3d17fa69448c0feb4e0c9d3f4c74a0324822e566a8340a56916d9d6f23cb3e85620334"; };
"semver@npm:7.3.8" = { filename = "semver-npm-7.3.8-25a996cb4f-ba9c7cbbf2.zip"; sha512 = "ba9c7cbbf2b7884696523450a61fee1a09930d888b7a8d7579025ad93d459b2d1949ee5bbfeb188b2be5f4ac163544c5e98491ad6152df34154feebc2cc337c1"; };
"tunnel-agent@npm:0.6.0" = { filename = "tunnel-agent-npm-0.6.0-64345ab7eb-05f6510358.zip"; sha512 = "05f6510358f8afc62a057b8b692f05d70c1782b70db86d6a1e0d5e28a32389e52fa6e7707b6c5ecccacc031462e4bc35af85ecfe4bbc341767917b7cf6965711"; };
"readable-stream@npm:3.6.0" = { filename = "readable-stream-npm-3.6.0-23a4a5eb56-d4ea81502d.zip"; sha512 = "d4ea81502d3799439bb955a3a5d1d808592cf3133350ed352aeaa499647858b27b1c4013984900238b0873ec8d0d8defce72469fb7a83e61d53f5ad61cb80dc8"; };
"color@npm:4.2.3" = { filename = "color-npm-4.2.3-4a23227581-0579629c02.zip"; sha512 = "0579629c02c631b426780038da929cca8e8d80a40158b09811a0112a107c62e10e4aad719843b791b1e658ab4e800558f2e87ca4522c8b32349d497ecb6adeb4"; };
"follow-redirects@npm:1.15.2" = { filename = "follow-redirects-npm-1.15.2-1ec1dd82be-faa66059b6.zip"; sha512 = "faa66059b66358ba65c234c2f2a37fcec029dc22775f35d9ad6abac56003268baf41e55f9ee645957b32c7d9f62baf1f0b906e68267276f54ec4b4c597c2b190"; };
"proxy-from-env@npm:1.1.0" = { filename = "proxy-from-env-npm-1.1.0-c13d07f26b-ed7fcc2ba0.zip"; sha512 = "ed7fcc2ba0a33404958e34d95d18638249a68c430e30fcb6c478497d72739ba64ce9810a24f53a7d921d0c065e5b78e3822759800698167256b04659366ca4d4"; };
"lodash.merge@npm:4.6.2" = { filename = "lodash.merge-npm-4.6.2-77cb4416bf-ad580b4bdb.zip"; sha512 = "ad580b4bdbb7ca1f7abf7e1bce63a9a0b98e370cf40194b03380a46b4ed799c9573029599caebc1b14e3f24b111aef72b96674a56cfa105e0f5ac70546cdc005"; };
"simple-get@npm:4.0.1" = { filename = "simple-get-npm-4.0.1-fa2a97645d-e4132fd27c.zip"; sha512 = "e4132fd27cf7af230d853fa45c1b8ce900cb430dd0a3c6d3829649fe4f2b26574c803698076c4006450efb0fad2ba8c5455fbb5755d4b0a5ec42d4f12b31d27e"; };
"ansi-regex@npm:5.0.1" = { filename = "ansi-regex-npm-5.0.1-c963a48615-2aa4bb54ca.zip"; sha512 = "2aa4bb54caf2d622f1afdad09441695af2a83aa3fe8b8afa581d205e57ed4261c183c4d3877cee25794443fde5876417d859c108078ab788d6af7e4fe52eb66b"; };
"telegram@npm:2.15.2" = { filename = "telegram-npm-2.15.2-af0dba7b70-1602a7bc74.zip"; sha512 = "1602a7bc74b47e752945eff9beb630f58db27172c5a9cda6a1ccf7b085c26470c9e7784a630309a97f680d25e87670f867d6847b5c1bd926f0b014ce70655af5"; };
"stream-parser@npm:0.3.1" = { filename = "stream-parser-npm-0.3.1-0b70187c85-4d86ff8cff.zip"; sha512 = "4d86ff8cffe7c7587dc91433fff9dce38a93ea7e9f47560055addc81eae6b6befab22b75643ce539faf325fe2b17d371778242566bed086e75f6cffb1e76c06c"; };
"tar-fs@npm:2.1.1" = { filename = "tar-fs-npm-2.1.1-e374d3b7a2-f5b9a70059.zip"; sha512 = "f5b9a70059f5b2969e65f037b4e4da2daf0fa762d3d232ffd96e819e3f94665dbbbe62f76f084f1acb4dbdcce16c6e4dac08d12ffc6d24b8d76720f4d9cf032d"; };
"node-gyp@npm:8.4.1" = { filename = "node-gyp-npm-8.4.1-13c90a9c9b-341710b5da.zip"; sha512 = "341710b5da39d3660e6a886b37e210d33f8282047405c2e62c277bcc744c7552c5b8b972ebc3a7d5c2813794e60cc48c3ebd142c46d6e0321db4db6c92dd0355"; };
"@tsconfig/node10@npm:1.0.8" = { filename = "@tsconfig-node10-npm-1.0.8-90a8cce25d-b8d5fffbc6.zip"; sha512 = "b8d5fffbc6b17ef64ef74f7fdbccee02a809a063ade785c3648dae59406bc207f70ea2c4296f92749b33019fa36a5ae716e42e49cc7f1bbf0fd147be0d6b970a"; };
"@tsconfig/node12@npm:1.0.9" = { filename = "@tsconfig-node12-npm-1.0.9-780563856d-a01b2400ab.zip"; sha512 = "a01b2400ab3582b86b589c6d31dcd0c0656f333adecde85d6d7d4086adb059808b82692380bb169546d189bf771ae21d02544a75b57bd6da4a5dd95f8567bec9"; };
"@tsconfig/node14@npm:1.0.1" = { filename = "@tsconfig-node14-npm-1.0.1-3ecac58e68-976345e896.zip"; sha512 = "976345e896c0f059867f94f8d0f6ddb8b1844fb62bf36b727de8a9a68f024857e5db97ed51d3325e23e0616a5e48c034ff51a8d595b3fe7e955f3587540489be"; };
"@tsconfig/node16@npm:1.0.2" = { filename = "@tsconfig-node16-npm-1.0.2-1f43ab567a-ca94d36397.zip"; sha512 = "ca94d3639714672bbfd55f03521d3f56bb6a25479bd425da81faf21f13e1e9d15f40f97377dedbbf477a5841c5b0c8f4cd1b391f33553d750b9202c54c2c07aa"; };
"acorn-walk@npm:8.2.0" = { filename = "acorn-walk-npm-8.2.0-2f2cac3177-1715e76c01.zip"; sha512 = "1715e76c01dd7b2d4ca472f9c58968516a4899378a63ad5b6c2d668bba8da21a71976c14ec5f5b75f887b6317c4ae0b897ab141c831d741dc76024d8745f1ad1"; };
"acorn@npm:8.7.0" = { filename = "acorn-npm-8.7.0-ca81d350ee-e0f79409d6.zip"; sha512 = "e0f79409d68923fbf1aa6d4166f3eedc47955320d25c89a20cc822e6ba7c48c5963d5bc657bc242d68f7a4ac9faf96eef033e8f73656da6c640d4219935fdfd0"; };
"arg@npm:4.1.3" = { filename = "arg-npm-4.1.3-1748b966a8-544af8dd3f.zip"; sha512 = "544af8dd3f60546d3e4aff084d451b96961d2267d668670199692f8d054f0415d86fc5497d0e641e91546f0aa920e7c29e5250e99fc89f5552a34b5d93b77f43"; };
"create-require@npm:1.1.1" = { filename = "create-require-npm-1.1.1-839884ca2e-a9a1503d43.zip"; sha512 = "a9a1503d4390d8b59ad86f4607de7870b39cad43d929813599a23714831e81c520bddf61bcdd1f8e30f05fd3a2b71ae8538e946eb2786dc65c2bbc520f692eff"; };
"diff@npm:4.0.2" = { filename = "diff-npm-4.0.2-73133c7102-f2c09b0ce4.zip"; sha512 = "f2c09b0ce4e6b301c221addd83bf3f454c0bc00caa3dd837cf6c127d6edf7223aa2bbe3b688feea110b7f262adbfc845b757c44c8a9f8c0c5b15d8fa9ce9d20d"; };
"make-error@npm:1.3.6" = { filename = "make-error-npm-1.3.6-ccb85d9458-b86e5e0e25.zip"; sha512 = "b86e5e0e25f7f777b77fabd8e2cbf15737972869d852a22b7e73c17623928fccb826d8e46b9951501d3f20e51ad74ba8c59ed584f610526a48f8ccf88aaec402"; };
"v8-compile-cache-lib@npm:3.0.1" = { filename = "v8-compile-cache-lib-npm-3.0.1-4886071ece-78089ad549.zip"; sha512 = "78089ad549e21bcdbfca10c08850022b22024cdcc2da9b168bcf5a73a6ed7bf01a9cebb9eac28e03cd23a684d81e0502797e88f3ccd27a32aeab1cfc44c39da0"; };
"yn@npm:3.1.1" = { filename = "yn-npm-3.1.1-8ad4259784-2c487b0e14.zip"; sha512 = "2c487b0e149e746ef48cda9f8bad10fc83693cd69d7f9dcd8be4214e985de33a29c9e24f3c0d6bcf2288427040a8947406ab27f7af67ee9456e6b84854f02dd6"; };
"delayed-stream@npm:1.0.0" = { filename = "delayed-stream-npm-1.0.0-c5a4c4cc02-46fe6e83e2.zip"; sha512 = "46fe6e83e2cb1d85ba50bd52803c68be9bd953282fa7096f51fc29edd5d67ff84ff753c51966061e5ba7cb5e47ef6d36a91924eddb7f3f3483b1c560f77a0020"; };
"mime-db@npm:1.52.0" = { filename = "mime-db-npm-1.52.0-b5371d6fd2-0d99a03585.zip"; sha512 = "0d99a03585f8b39d68182803b12ac601d9c01abfa28ec56204fa330bc9f3d1c5e14beb049bafadb3dbdf646dfb94b87e24d4ec7b31b7279ef906a8ea9b6a513f"; };
"date-format@npm:4.0.3" = { filename = "date-format-npm-4.0.3-ce15085cfe-8ae4d9de35.zip"; sha512 = "8ae4d9de3532010169a89bc7b079342051ba3ec88552636aa677bfb53e8eb15113af8394679aea7d41367dc8bb6e9865da17f21ac2802202180b09d6e3f2339e"; };
"flatted@npm:3.2.5" = { filename = "flatted-npm-3.2.5-0ee5a8875f-3c436e9695.zip"; sha512 = "3c436e9695ccca29620b4be5671dd72e5dd0a7500e0856611b7ca9bd8169f177f408c3b9abfa78dfe1493ee2d873e2c119080a8a9bee4e1a186a9e60ca6c89f1"; };
"safe-buffer@npm:5.2.1" = { filename = "safe-buffer-npm-5.2.1-3481c8aa9b-b99c4b41fd.zip"; sha512 = "b99c4b41fdd67a6aaf280fcd05e9ffb0813654894223afb78a31f14a19ad220bba8aba1cb14eddce1fcfb037155fe6de4e861784eb434f7d11ed58d1e70dd491"; };
"@cspotcode/source-map-support@npm:0.8.1" = { filename = "@cspotcode-source-map-support-npm-0.8.1-964f2de99d-5718f26708.zip"; sha512 = "5718f267085ed8edb3e7ef210137241775e607ee18b77d95aa5bd7514f47f5019aa2d82d96b3bf342ef7aa890a346fa1044532ff7cc3009e7d24fce3ce6200fa"; };
"inherits@npm:2.0.4" = { filename = "inherits-npm-2.0.4-c66b3957a0-4a48a73384.zip"; sha512 = "4a48a733847879d6cf6691860a6b1e3f0f4754176e4d71494c41f3475553768b10f84b5ce1d40fbd0e34e6bfbb864ee35858ad4dd2cf31e02fc4a154b724d7f1"; };
"util-deprecate@npm:1.0.2" = { filename = "util-deprecate-npm-1.0.2-e3fe1a219c-474acf1146.zip"; sha512 = "474acf1146cb2701fe3b074892217553dfcf9a031280919ba1b8d651a068c9b15d863b7303cb15bd00a862b498e6cf4ad7b4a08fb134edd5a6f7641681cb54a2"; };
"fs-extra@npm:8.1.0" = { filename = "fs-extra-npm-8.1.0-197473387f-bf44f0e6ce.zip"; sha512 = "bf44f0e6cea59d5ce071bba4c43ca76d216f89e402dc6285c128abc0902e9b8525135aa808adad72c9d5d218e9f4bcc63962815529ff2f684ad532172a284880"; };
"debug@npm:4.3.3" = { filename = "debug-npm-4.3.3-710fd4cc7f-14472d56fe.zip"; sha512 = "14472d56fe4a94dbcfaa6dbed2dd3849f1d72ba78104a1a328047bb564643ca49df0224c3a17fa63533fd11dd3d4c8636cd861191232a2c6735af00cc2d4de16"; };
"needle@npm:2.9.1" = { filename = "needle-npm-2.9.1-f40e591726-746ae3a378.zip"; sha512 = "746ae3a3782f0a057ff304a98843cc6f2009f978a0fad0c3e641a9d46d0b5702bb3e197ba08aecd48678067874a991c4f5fc320c7e51a4c041d9dd3441146cf0"; };
"lru-cache@npm:6.0.0" = { filename = "lru-cache-npm-6.0.0-b4c8668fe1-f97f499f89.zip"; sha512 = "f97f499f898f23e4585742138a22f22526254fdba6d75d41a1c2526b3b6cc5747ef59c5612ba7375f42aca4f8461950e925ba08c991ead0651b4918b7c978297"; };
"string_decoder@npm:1.3.0" = { filename = "string_decoder-npm-1.3.0-2422117fd0-8417646695.zip"; sha512 = "8417646695a66e73aefc4420eb3b84cc9ffd89572861fe004e6aeb13c7bc00e2f616247505d2dbbef24247c372f70268f594af7126f43548565c68c117bdeb56"; };
"quote-api@https://github.com/Clansty/quote-api/archive/8d2a51e512a6d33e8b5948e39c7c9f3d886bc3d6.tar.gz" = { filename = "quote-api-https-e7bbf7252b-f8500c306f.zip"; sha512 = "f8500c306f1a601e326026d120adfc8933350e0840e8d870703c759689156383e4a0fb98b121333fb5f11160413a055dc88a2dcabf3152f0b1d42f1dcf87012a"; };
"color-convert@npm:2.0.1" = { filename = "color-convert-npm-2.0.1-79730e935b-79e6bdb9fd.zip"; sha512 = "79e6bdb9fd479a205c71d89574fccfb22bd9053bd98c6c4d870d65c132e5e904e6034978e55b43d69fcaa7433af2016ee203ce76eeba9cfa554b373e7f7db336"; };
"simple-concat@npm:1.0.1" = { filename = "simple-concat-npm-1.0.1-48df70de29-4d211042cc.zip"; sha512 = "4d211042cc3d73a718c21ac6c4e7d7a0363e184be6a5ad25c8a1502e49df6d0a0253979e3d50dbdd3f60ef6c6c58d756b5d66ac1e05cda9cacd2e9fc59e3876a"; };
"@cryptography/aes@npm:0.1.1" = { filename = "@cryptography-aes-npm-0.1.1-0bf958ef1c-81a6b93d6d.zip"; sha512 = "81a6b93d6d16413c7dda5f160b845add49bb6df6c314b7260312627fcbb7130b7ed7f51b063de0f712439b0c30b1c3eac56cb78ed180abaab3fd682322b4ed80"; };
"big-integer@npm:1.6.51" = { filename = "big-integer-npm-1.6.51-1a244d8e1f-3d444173d1.zip"; sha512 = "3d444173d1b2e20747e2c175568bedeebd8315b0637ea95d75fd27830d3b8e8ba36c6af40374f36bdaea7b5de376dcada1b07587cb2a79a928fccdb6e6e3c518"; };
"mime@npm:3.0.0" = { filename = "mime-npm-3.0.0-8d911e4c06-f43f9b7bfa.zip"; sha512 = "f43f9b7bfa64534e6b05bd6062961681aeb406a5b53673b53b683f27fcc4e739989941836a355eef831f4478923651ecc739f4a5f6e20a76487b432bfd4db928"; };
"pako@npm:2.0.4" = { filename = "pako-npm-2.0.4-1d4e28f3ac-82b9b0b99d.zip"; sha512 = "82b9b0b99dd830c9103856a6dbd10f0cb2c8c32b9768184727ea381a99666de9a47a069d2e6efe6acf09336f363956b50835c196ef9311b34b7274d420eb0d88"; };
"path-browserify@npm:1.0.1" = { filename = "path-browserify-npm-1.0.1-f975d99a99-c6d7fa3764.zip"; sha512 = "c6d7fa376423fe35b95b2d67990060c3ee304fc815ff0a2dc1c6c3cfaff2bd0d572ee67e18f19d0ea3bbe32e8add2a05021132ac40509416459fffee35200699"; };
"real-cancellable-promise@npm:1.1.1" = { filename = "real-cancellable-promise-npm-1.1.1-9646e6b10e-8bc4d7e973.zip"; sha512 = "8bc4d7e9739f5db4cc20168e26deef6cbccaf52c857f65b58c30c166649fca230c20354a714fa8262687857f2cc8b6f79ac7d19b6fe5006298ffc5782448f58a"; };
"store2@npm:2.14.2" = { filename = "store2-npm-2.14.2-c286d27e4f-6f270fc5ba.zip"; sha512 = "6f270fc5bab99b63f45fcc7bd8b99c2714b4adf880f557ed7ffb5ed3987131251165bccde425a00928aaf044870aee79ddeef548576d093c68703ed2edec45d7"; };
"ts-custom-error@npm:3.2.0" = { filename = "ts-custom-error-npm-3.2.0-7ef8105f26-b1d2016de8.zip"; sha512 = "b1d2016de891cecea46a70c4adbaf1e6f7c6178994f72070ee3c218cc1a9e2b12c3011defb8385f15dc03ddfafd8d88ef2c9217e23700fb6e537755f33b9c801"; };
"glob@npm:7.2.0" = { filename = "glob-npm-7.2.0-bb4644d239-78a8ea9423.zip"; sha512 = "78a8ea942331f08ed2e055cb5b9e40fe6f46f579d7fd3d694f3412fe5db23223d29b7fee1575440202e9a7ff9a72ab106a39fee39934c7bedafe5e5f8ae20134"; };
"streamroller@npm:3.0.2" = { filename = "streamroller-npm-3.0.2-6d7ba8035a-1f323824f0.zip"; sha512 = "1f323824f0e81cc085c24f33addfd8ef00d0c15aafee520a8cf207ca6e2dc674fd852528c7b4450cc87f4335d1269ed18b3f0188853d45d7f0912c9a205d1fc1"; };
"color-string@npm:1.9.0" = { filename = "color-string-npm-1.9.0-75382c0441-93c6678b84.zip"; sha512 = "93c6678b847f8cfa47d19677fd19e1d4b19d7a33f100644400357c298266080b5bca64e5f874fa8ac8cc0aa0606ad44f7a838b4e6fd05e6affea190a68555bb4"; };
"decompress-response@npm:6.0.0" = { filename = "decompress-response-npm-6.0.0-359de2878c-d377cf47e0.zip"; sha512 = "d377cf47e02d805e283866c3f50d3d21578b779731e8c5072d6ce8c13cc31493db1c2f6784da9d1d5250822120cefa44f1deab112d5981015f2e17444b763812"; };
"once@npm:1.4.0" = { filename = "once-npm-1.4.0-ccf03ef07a-cd0a885013.zip"; sha512 = "cd0a88501333edd640d95f0d2700fbde6bff20b3d4d9bdc521bdd31af0656b5706570d6c6afe532045a20bb8dc0849f8332d6f2a416e0ba6d3d3b98806c7db68"; };
"chownr@npm:1.1.4" = { filename = "chownr-npm-1.1.4-5bd400ab08-115648f8eb.zip"; sha512 = "115648f8eb38bac5e41c3857f3e663f9c39ed6480d1349977c4d96c95a47266fcacc5a5aabf3cb6c481e22d72f41992827db47301851766c4fd77ac21a4f081d"; };
"mkdirp-classic@npm:0.5.3" = { filename = "mkdirp-classic-npm-0.5.3-3b5c991910-3f4e088208.zip"; sha512 = "3f4e088208270bbcc148d53b73e9a5bd9eef05ad2cbf3b3d0ff8795278d50dd1d11a8ef1875ff5aea3fa888931f95bfcb2ad5b7c1061cfefd6284d199e6776ac"; };
"env-paths@npm:2.2.1" = { filename = "env-paths-npm-2.2.1-7c7577428c-65b5df55a8.zip"; sha512 = "65b5df55a8bab92229ab2b40dad3b387fad24613263d103a97f91c9fe43ceb21965cd3392b1ccb5d77088021e525c4e0481adb309625d0cb94ade1d1fb8dc17e"; };
"graceful-fs@npm:4.2.9" = { filename = "graceful-fs-npm-4.2.9-ee48e00aaa-68ea4e07ff.zip"; sha512 = "68ea4e07ff2c041ada184f9278b830375f8e0b75154e3f080af6b70f66172fabb4108d19b3863a96b53fc068a310b9b6493d86d1291acc5f3861eb4b79d26ad6"; };
"async-mutex@npm:0.3.2" = { filename = "async-mutex-npm-0.3.2-600f6c46a1-620b771dfd.zip"; sha512 = "620b771dfdea1cad0a6b712915c31a1e3ca880a8cf1eae92b4590f435995e0260929c6ebaae0b9126b1456790ea498064b5bb9a506948cda760f48d3d0dcc4c8"; };
"node-localstorage@npm:2.2.1" = { filename = "node-localstorage-npm-2.2.1-bc01536b9e-c6b2637e1c.zip"; sha512 = "c6b2637e1c0957dcd78581de104530be57600c8b9fd13af50a61203e6ffa1f75ddd931162b2afd85d45c599f4e153811e27f1039748da89226edcc951f250fb8"; };
"debug@npm:2.6.9" = { filename = "debug-npm-2.6.9-7d4cb597dc-d2f51589ca.zip"; sha512 = "d2f51589ca66df60bf36e1fa6e4386b318c3f1e06772280eea5b1ae9fd3d05e9c2b7fd8a7d862457d00853c75b00451aa2d7459b924629ee385287a650f58fe6"; };
"nopt@npm:5.0.0" = { filename = "nopt-npm-5.0.0-304b40fbfe-d35fdec187.zip"; sha512 = "d35fdec187269503843924e0114c0c6533fb54bbf1620d0f28b4b60ba01712d6687f62565c55cc20a504eff0fbe5c63e22340c3fad549ad40469ffb611b04f2f"; };
"semver@npm:7.3.5" = { filename = "semver-npm-7.3.5-618cf5db6a-5eafe6102b.zip"; sha512 = "5eafe6102bea2a7439897c1856362e31cc348ccf96efd455c8b5bc2c61e6f7e7b8250dc26b8828c1d76a56f818a7ee907a36ae9fb37a599d3d24609207001d60"; };
"which@npm:2.0.2" = { filename = "which-npm-2.0.2-320ddf72f7-1a5c563d3c.zip"; sha512 = "1a5c563d3c1b52d5f893c8b61afe11abc3bab4afac492e8da5bde69d550de701cf9806235f20a47b5c8fa8a1d6a9135841de2596535e998027a54589000e66d1"; };
"buffer@npm:6.0.3" = { filename = "buffer-npm-6.0.3-cd90dfedfe-5ad23293d9.zip"; sha512 = "5ad23293d9a731e4318e420025800b42bf0d264004c0286c8cc010af7a270c7a0f6522e84f54b9ad65cbd6db20b8badbfd8d2ebf4f80fa03dab093b89e68c3f9"; };
"bufferutil@npm:4.0.6" = { filename = "bufferutil-npm-4.0.6-b93c8a5e05-dd10756094.zip"; sha512 = "dd107560947445280af7820c3d0534127b911577d85d537e1d7e0aa30fd634853cef8a994d6e8aed3d81388ab1a20257de776164afe6a6af8e78f5f17968ebd6"; };
"socks@npm:2.7.0" = { filename = "socks-npm-2.7.0-cc1cb019db-0b5d94e2b3.zip"; sha512 = "0b5d94e2b3c11e7937b40fc5dac1e80d8b92a330e68c51f1d271ce6980c70adca42a3f8cd47c4a5769956bada074823b53374f2dc5f2ea5c2121b222dec6eadf"; };
"utf-8-validate@npm:5.0.8" = { filename = "utf-8-validate-npm-5.0.8-a36435e0ec-cb1be3fa4e.zip"; sha512 = "cb1be3fa4eb896be17945a2e46c25f47ef9344d5955703a09d9d831efef681ce120bddfe02a8ebf3a96580ffa8f70edda55623e4d021adff70cb81cd0c8a885e"; };
"universalify@npm:0.1.2" = { filename = "universalify-npm-0.1.2-9b22d31d2d-40cdc60f6e.zip"; sha512 = "40cdc60f6e61070fe658ca36016a8f4ec216b29bf04a55dce14e3710cc84c7448538ef4dad3728d0bfe29975ccd7bfb5f414c45e7b78883567fb31b246f02dff"; };
"sax@npm:1.2.4" = { filename = "sax-npm-1.2.4-178f05f12f-d3df7d32b8.zip"; sha512 = "d3df7d32b897a2c2f28e941f732c71ba90e27c24f62ee918bd4d9a8cfb3553f2f81e5493c7f0be94a11c1911b643a9108f231dd6f60df3fa9586b5d2e3e9e1fe"; };
"yallist@npm:4.0.0" = { filename = "yallist-npm-4.0.0-b493d9e907-343617202a.zip"; sha512 = "343617202af32df2a15a3be36a5a8c0c8545208f3d3dfbc6bb7c3e3b7e8c6f8e7485432e4f3b88da3031a6e20afa7c711eded32ddfb122896ac5d914e75848d5"; };
"pump@npm:3.0.0" = { filename = "pump-npm-3.0.0-0080bf6a7a-e42e9229fb.zip"; sha512 = "e42e9229fba14732593a718b04cb5e1cfef8254544870997e0ecd9732b189a48e1256e4e5478148ecb47c8511dca2b09eae56b4d0aad8009e6fac8072923cfc9"; };
"dotenv@npm:7.0.0" = { filename = "dotenv-npm-7.0.0-9fbf3b4fd8-18a7b3ef0e.zip"; sha512 = "18a7b3ef0e90fd6fcce7c7cbdd48d923b0cb180807540b80c797bda4a098097e17820d6315ae28eec22f73954cd0ab9d81904d46370183817c09f694d40566ff"; };
"emoji-db@npm:14.0.1" = { filename = "emoji-db-npm-14.0.1-ddd447583f-c16918c3a5.zip"; sha512 = "c16918c3a56a86bccc990445c067ef858b60bee3e38e7d6456d46ee297eb3a11e99fb142fb9425bc29947680c7892264bb082b9dc90a9260d09ab9d59c0ca443"; };
"koa-response-time@npm:2.1.0" = { filename = "koa-response-time-npm-2.1.0-008d050cd4-c51c0e29aa.zip"; sha512 = "c51c0e29aa2c8863dfbce422565908b0356c04cab3fa6971c541ab222f3bb24a913ea00b2111cea13fd6b895cd95d4219e05b128bee90289eb2dc90f629b3b73"; };
"lottie-node@npm:2.0.0" = { filename = "lottie-node-npm-2.0.0-e20ab55b19-611a9896ba.zip"; sha512 = "611a9896ba25d3644b1651ba7d3799c04553df9b906548513583f9b99d785c91541562489fbc945445d0ddc0513e07a2c0d21a1acf191eeaf54061ef8279e55c"; };
"lottie-web@npm:5.10.1" = { filename = "lottie-web-npm-5.10.1-b546f02492-5da7d0804b.zip"; sha512 = "5da7d0804b4b895d795e3eb299e45a0dfd1c7a5268784668ff72965d4c6e0806e0b60984fade74dd681909cc325d45b13d4a8e24c96e73472a00646e16b628b4"; };
"runes@npm:0.4.3" = { filename = "runes-npm-0.4.3-2276541661-8bb7a16eb7.zip"; sha512 = "8bb7a16eb75799fe8a3a9cb06fa622f3ee3e3d1116897ac48e8238fc40aeba74d77232d6a7b1afedc9fa96fd6075b4dbbed6c85d4e66df9b727c5056225fc177"; };
"jsonfile@npm:4.0.0" = { filename = "jsonfile-npm-4.0.0-10ce3aea15-6447d6224f.zip"; sha512 = "6447d6224f0d31623eef9b51185af03ac328a7553efcee30fa423d98a9e276ca08db87d71e17f2310b0263fd3ffa6c2a90a6308367f661dc21580f9469897c9e"; };
"debug@npm:3.2.7" = { filename = "debug-npm-3.2.7-754e818c7a-b3d8c59407.zip"; sha512 = "b3d8c5940799914d30314b7c3304a43305fd0715581a919dacb8b3176d024a782062368405b47491516d2091d6462d4d11f2f4974a405048094f8bfebfa3071c"; };
"iconv-lite@npm:0.4.24" = { filename = "iconv-lite-npm-0.4.24-c5c4ac6695-bd9f120f5a.zip"; sha512 = "bd9f120f5a5b306f0bc0b9ae1edeb1577161503f5f8252a20f1a9e56ef8775c9959fd01c55f2d3a39d9a8abaf3e30c1abeb1895f367dcbbe0a8fd1c9ca01c4f6"; };
"color-name@npm:1.1.4" = { filename = "color-name-npm-1.1.4-025792b0ea-b044585952.zip"; sha512 = "b0445859521eb4021cd0fb0cc1a75cecf67fceecae89b63f62b201cca8d345baf8b952c966862a9d9a2632987d4f6581f0ec8d957dfacece86f0a7919316f610"; };
"@jridgewell/trace-mapping@npm:0.3.9" = { filename = "@jridgewell-trace-mapping-npm-0.3.9-91625cd7fb-d89597752f.zip"; sha512 = "d89597752fd88d3f3480845691a05a44bd21faac18e2185b6f436c3b0fd0c5a859fbbd9aaa92050c4052caf325ad3e10e2e1d1b64327517471b7d51babc0ddef"; };
"lru-cache@npm:5.1.1" = { filename = "lru-cache-npm-5.1.1-f475882a51-c154ae1cbb.zip"; sha512 = "c154ae1cbb0c2206d1501a0e94df349653c92c8cbb25236d7e85190bcaf4567a03ac6eb43166fabfa36fd35623694da7233e88d9601fbf411a9a481d85dbd2cb"; };
"object-sizeof@npm:1.6.3" = { filename = "object-sizeof-npm-1.6.3-ff0c5cc6ce-a1195e0e0c.zip"; sha512 = "a1195e0e0caa80504e7ff20da54c73b4ed57c1f36482743436c7014c615bdc1528f3751e928bb9efa6f1960cd7f039aeb44b19f9dda63686b9995e1bbe4538af"; };
"smartcrop-sharp@npm:2.0.7" = { filename = "smartcrop-sharp-npm-2.0.7-9d35f8b5a5-571e36c3c9.zip"; sha512 = "571e36c3c99c9642d4b425d9b666ccd2ca07e526ef08302cc177cf9b54a339c7f874faff36d758039b8345a33a93877b00bbadb34c2ccef4f6a7b89ffbf95d23"; };
"htmlparser2@npm:6.1.0" = { filename = "htmlparser2-npm-6.1.0-4ef89ab31e-81a7b3d9c3.zip"; sha512 = "81a7b3d9c3bb9acb568a02fc9b1b81ffbfa55eae7f1c41ae0bf840006d1dbf54cb3aa245b2553e2c94db674840a9f0fdad7027c9a9d01a062065314039058c4e"; };
"fs.realpath@npm:1.0.0" = { filename = "fs.realpath-npm-1.0.0-c8f05d8126-99ddea01a7.zip"; sha512 = "99ddea01a7e75aa276c250a04eedeffe5662bce66c65c07164ad6264f9de18fb21be9433ead460e54cff20e31721c811f4fb5d70591799df5f85dce6d6746fd0"; };
"path-is-absolute@npm:1.0.1" = { filename = "path-is-absolute-npm-1.0.1-31bc695ffd-060840f92c.zip"; sha512 = "060840f92cf8effa293bcc1bea81281bd7d363731d214cbe5c227df207c34cd727430f70c6037b5159c8a870b9157cba65e775446b0ab06fd5ecc7e54615a3b8"; };
"mimic-response@npm:3.1.0" = { filename = "mimic-response-npm-3.1.0-a4a24b4e96-25739fee32.zip"; sha512 = "25739fee32c17f433626bf19f016df9036b75b3d84a3046c7d156e72ec963dd29d7fc8a302f55a3d6c5a4ff24259676b15d915aad6480815a969ff2ec0836867"; };
"wrappy@npm:1.0.2" = { filename = "wrappy-npm-1.0.2-916de4d4b3-159da4805f.zip"; sha512 = "159da4805f7e84a3d003d8841557196034155008f817172d4e986bd591f74aa82aa7db55929a54222309e01079a65a92a9e6414da5a6aa4b01ee44a511ac3ee5"; };
"npmlog@npm:6.0.1" = { filename = "npmlog-npm-6.0.1-f597f2e057-f1a4078a73.zip"; sha512 = "f1a4078a73ebc89896a832bbf869f491c32ecb12e0434b9a7499878ce8f29f22e72befe3c53cd8cdc9dbf4b4057297e783ab0b6746a8b067734de6205af4d538"; };
"tslib@npm:2.3.1" = { filename = "tslib-npm-2.3.1-0e21e18015-de17a98d46.zip"; sha512 = "de17a98d4614481f7fcb5cd53ffc1aaf8654313be0291e1bfaee4b4bb31a20494b7d218ff2e15017883e8ea9626599b3b0e0229c18383ba9dce89da2adf15cb9"; };
"ms@npm:2.0.0" = { filename = "ms-npm-2.0.0-9e1101a471-0e6a22b8b7.zip"; sha512 = "0e6a22b8b746d2e0b65a430519934fefd41b6db0682e3477c10f60c76e947c4c0ad06f63ffdf1d78d335f83edee8c0aa928aa66a36c7cd95b69b26f468d527f4"; };
"koa-bodyparser@npm:4.3.0" = { filename = "koa-bodyparser-npm-4.3.0-1bf6ca26ab-c227fe0fb5.zip"; sha512 = "c227fe0fb5a55b98fc91d865e80229b60178d216d53b732b07833eb38f48a7ed6aa768a083bc06e359db33298547e9a65842fbe9d3f0fdaf5149fe0becafc88f"; };
"prebuild-install@npm:7.1.1" = { filename = "prebuild-install-npm-7.1.1-cb9fc9d341-dbf96d0146.zip"; sha512 = "dbf96d0146b6b5827fc8f67f72074d2e19c69628b9a7a0a17d0fad1bf37e9f06922896972e074197fc00a52eae912993e6ef5a0d471652f561df5cb516f3f467"; };
"minimatch@npm:3.1.1" = { filename = "minimatch-npm-3.1.1-38a381be52-e9e3772e4e.zip"; sha512 = "e9e3772e4ea06ea3a888d39bc7690d3c812ee7e5a70c2d2f568ccadac0249a027f865589d19ad03ed937e6ca3b4ad35f85411db9670f7877d8fc2ed452f1cd37"; };
"simple-swizzle@npm:0.2.2" = { filename = "simple-swizzle-npm-0.2.2-8dee37fad1-a7f3f2ab5c.zip"; sha512 = "a7f3f2ab5c76c4472d5c578df892e857323e452d9f392e1b5cf74b74db66e6294a1e1b8b390b519fa1b96b5b613f2a37db6cffef52c3f1f8f3c5ea64eb2d54c0"; };
"abbrev@npm:1.1.1" = { filename = "abbrev-npm-1.1.1-3659247eab-a4a97ec07d.zip"; sha512 = "a4a97ec07d7ea112c517036882b2ac22f3109b7b19077dc656316d07d308438aac28e4d9746dc4d84bf6b1e75b4a7b0a5f3cb30592419f128ca9a8cee3bcfa17"; };
"tar-stream@npm:2.2.0" = { filename = "tar-stream-npm-2.2.0-884c79b510-699831a8b9.zip"; sha512 = "699831a8b97666ef50021c767f84924cfee21c142c2eb0e79c63254e140e6408d6d55a065a2992548e72b06de39237ef2b802b99e3ece93ca3904a37622a66f3"; };
"base64-js@npm:1.5.1" = { filename = "base64-js-npm-1.5.1-b2f7275641-669632eb37.zip"; sha512 = "669632eb3745404c2f822a18fc3a0122d2f9a7a13f7fb8b5823ee19d1d2ff9ee5b52c53367176ea4ad093c332fd5ab4bd0ebae5a8e27917a4105a4cfc86b1005"; };
"node-gyp-build@npm:4.3.0" = { filename = "node-gyp-build-npm-4.3.0-87bdf5216f-1ecab16d9f.zip"; sha512 = "1ecab16d9f275174d516e223f60f65ebe07540347d5c04a6a7d6921060b7f2e3af4f19463d9d1dcedc452e275c2ae71354a99405e55ebd5b655bb2f38025c728"; };
"ip@npm:2.0.0" = { filename = "ip-npm-2.0.0-204facb3cc-cfcfac6b87.zip"; sha512 = "cfcfac6b873b701996d71ec82a7dd27ba92450afdb421e356f44044ed688df04567344c36cbacea7d01b1c39a4c732dc012570ebe9bebfb06f27314bca625349"; };
"smart-buffer@npm:4.2.0" = { filename = "smart-buffer-npm-4.2.0-5ac3f668bb-b5167a7142.zip"; sha512 = "b5167a7142c1da704c0e3af85c402002b597081dd9575031a90b4f229ca5678e9a36e8a374f1814c8156a725d17008ae3bde63b92f9cfd132526379e580bec8b"; };
"canvas@https://github.com/Automattic/node-canvas.git#commit=198080580a0e3938c48daae357b88a1638a9ddcd" = { filename = "canvas-https-cbcd64979e-54e6153a42.zip"; sha512 = "54e6153a42d01a5f030180c95d8a825d12d6b352717cbafdc9760e5f9424d9cb023d86c94f93b089f2581149ea88befaafa56d66e6e4c1c84e3e3a6a0323a66d"; };
"websocket@npm:1.0.34" = { filename = "websocket-npm-1.0.34-3aaa6c5dc0-8a0ce6d79c.zip"; sha512 = "8a0ce6d79cc1334bb6ea0d607f0092f3d32700b4dd19e4d5540f2a85f3b50e1f8110da0e4716737056584dde70bbebcb40bbd94bbb437d7468c71abfbfa077d8"; };
"inflight@npm:1.0.6" = { filename = "inflight-npm-1.0.6-ccedb4b908-f4f76aa072.zip"; sha512 = "f4f76aa072ce19fae87ce1ef7d221e709afb59d445e05d47fba710e85470923a75de35bfae47da6de1b18afc3ce83d70facf44cfb0aff89f0a3f45c0a0244dfd"; };
"tar@npm:6.1.11" = { filename = "tar-npm-6.1.11-e6ac3cba9c-a04c07bb9e.zip"; sha512 = "a04c07bb9e2d8f46776517d4618f2406fb977a74d914ad98b264fc3db0fe8224da5bec11e5f8902c5b9bcb8ace22d95fbe3c7b36b8593b7dfc8391a25898f32f"; };
"ms@npm:2.1.3" = { filename = "ms-npm-2.1.3-81ff3cfac1-aa92de6080.zip"; sha512 = "aa92de608021b242401676e35cfa5aa42dd70cbdc082b916da7fb925c542173e36bce97ea3e804923fe92c0ad991434e4a38327e15a1b5b5f945d66df615ae6d"; };
"safer-buffer@npm:2.1.2" = { filename = "safer-buffer-npm-2.1.2-8d5c0b705e-cab8f25ae6.zip"; sha512 = "cab8f25ae6f1434abee8d80023d7e72b598cf1327164ddab31003c51215526801e40b66c5e65d658a0af1e9d6478cadcb4c745f4bd6751f97d8644786c0978b0"; };
"koa-logger@npm:3.2.1" = { filename = "koa-logger-npm-3.2.1-c647362edb-b29ba25eb4.zip"; sha512 = "b29ba25eb433452bfda48e51acd5d206128411966acc09bb13ce3a0cec9192f78bb27e23efd615d0e7f46eeb2588ee8d2541d72665a4aa18d27a177e78dca909"; };
"koa-ratelimit@npm:4.3.0" = { filename = "koa-ratelimit-npm-4.3.0-632b6ab913-54d3235941.zip"; sha512 = "54d3235941319c3938b5f40c0837eab12b1c25a0d182f439d7b0b0324dbd20a3085121bef842172358d6505a9ec1c1dad42e64920ce8c43a19272584b60400e4"; };
"fs-extra@npm:10.0.0" = { filename = "fs-extra-npm-10.0.0-4f8c704115-5285a3d8f3.zip"; sha512 = "5285a3d8f34b917cf2b66af8c231a40c1623626e9d701a20051d3337be16c6d7cac94441c8b3732d47a92a2a027886ca93c69b6a4ae6aee3c89650d2a8880c0a"; };
"end-of-stream@npm:1.4.4" = { filename = "end-of-stream-npm-1.4.4-497fc6dee1-530a5a5a1e.zip"; sha512 = "530a5a5a1e517e962854a31693dbb5c0b2fc40b46dad2a56a2deec656ca040631124f4795823acc68238147805f8b021abbe221f4afed5ef3c8e8efc2024908b"; };
"@jridgewell/resolve-uri@npm:3.1.0" = { filename = "@jridgewell-resolve-uri-npm-3.1.0-6ff2351e61-b5ceaaf9a1.zip"; sha512 = "b5ceaaf9a110fcb2780d1d8f8d4a0bfd216702f31c988d8042e5f8fbe353c55d9b0f55a1733afdc64806f8e79c485d2464680ac48a0d9fcadb9548ee6b81d267"; };
"@jridgewell/sourcemap-codec@npm:1.4.14" = { filename = "@jridgewell-sourcemap-codec-npm-1.4.14-f5f0630788-61100637b6.zip"; sha512 = "61100637b6d173d3ba786a5dff019e1a74b1f394f323c1fee337ff390239f053b87266c7a948777f4b1ee68c01a8ad0ab61e5ff4abb5a012a0b091bec391ab97"; };
"yallist@npm:3.1.1" = { filename = "yallist-npm-3.1.1-a568a556b4-48f7bb00dc.zip"; sha512 = "48f7bb00dc19fc635a13a39fe547f527b10c9290e7b3e836b9a8f1ca04d4d342e85714416b3c2ab74949c9c66f9cebb0473e6bc353b79035356103b47641285d"; };
"smartcrop@npm:2.0.5" = { filename = "smartcrop-npm-2.0.5-4e19562578-42828b1c14.zip"; sha512 = "42828b1c14ef1e9e6b86c0267517f1e444880b418dc1d4033ef45a860d06e0f00bee15f02f46300c6e1c97c48e5001c11375eaba692a00761ef37ccd95e34e05"; };
"domelementtype@npm:2.2.0" = { filename = "domelementtype-npm-2.2.0-c37b3b15bf-24cb386198.zip"; sha512 = "24cb386198640cd58aa36f8c987f2ea61859929106d06ffcc8f547e70cb2ed82a6dc56dcb8252b21fba1f1ea07df6e4356d60bfe57f77114ca1aed6828362629"; };
"entities@npm:2.2.0" = { filename = "entities-npm-2.2.0-0fc8d5b2f7-19010dacaf.zip"; sha512 = "19010dacaf0912c895ea262b4f6128574f9ccf8d4b3b65c7e8334ad0079b3706376360e28d8843ff50a78aabcb8f08f0a32dbfacdc77e47ed77ca08b713669b3"; };
"write-file-atomic@npm:1.3.4" = { filename = "write-file-atomic-npm-1.3.4-485bffebbc-ed5a1fc812.zip"; sha512 = "ed5a1fc812abe99dd03cd42be145a5e13af42609e19873244510c71b2e261a169db6d06cb9a92b6b0489dd05f045e53bf464d9ad7713e443934af9b81a9cc65d"; };
"jimp@npm:0.16.2" = { filename = "jimp-npm-0.16.2-d033c62f70-da1ee6a6e5.zip"; sha512 = "da1ee6a6e5cabc9dc86c68542e3d21ac1e485162811ff59105db972fc7a73ed07c5f77f996a8095a34b826aab54c786d2958205c82b05d39cdc67f4af0ca853f"; };
"console-control-strings@npm:1.1.0" = { filename = "console-control-strings-npm-1.1.0-e3160e5275-8755d76787.zip"; sha512 = "8755d76787f94e6cf79ce4666f0c5519906d7f5b02d4b884cf41e11dcd759ed69c57da0670afd9236d229a46e0f9cf519db0cd829c6dca820bb5a5c3def584ed"; };
"set-blocking@npm:2.0.0" = { filename = "set-blocking-npm-2.0.0-49e2cffa24-6e65a05f7c.zip"; sha512 = "6e65a05f7cf7ebdf8b7c75b101e18c0b7e3dff4940d480efed8aad3a36a4005140b660fa1d804cb8bce911cac290441dc728084a30504d3516ac2ff7ad607b02"; };
"copy-to@npm:2.0.1" = { filename = "copy-to-npm-2.0.1-474b7b678b-05ea12875b.zip"; sha512 = "05ea12875bdc96ae053a3b30148e9d992026035ff2bfcc0b615e8d49d1cf8fc3d1f40843f9a4b7b1b6d9118eeebcba31e621076d7de525828aa9c07d22a81dab"; };
"expand-template@npm:2.0.3" = { filename = "expand-template-npm-2.0.3-80de959306-588c198472.zip"; sha512 = "588c19847216421ed92befb521767b7018dc88f88b0576df98cb242f20961425e96a92cbece525ef28cc5becceae5d544ae0f5b9b5e2aa05acb13716ca5b3099"; };
"github-from-package@npm:0.0.0" = { filename = "github-from-package-npm-0.0.0-519f80c9a1-14e448192a.zip"; sha512 = "14e448192a35c1e42efee94c9d01a10f42fe790375891a24b25261246ce9336ab9df5d274585aedd4568f7922246c2a78b8a8cd2571bfe99c693a9718e7dd0e3"; };
"minimist@npm:1.2.5" = { filename = "minimist-npm-1.2.5-ced0e1f617-86706ce5b3.zip"; sha512 = "86706ce5b36c16bfc35c5fe3dbb01d5acdc9a22f2b6cc810b6680656a1d2c0e44a0159c9a3ba51fb072bb5c203e49e10b51dcd0eec39c481f4c42086719bae52"; };
"napi-build-utils@npm:1.0.2" = { filename = "napi-build-utils-npm-1.0.2-892e4bba56-06c14271ee.zip"; sha512 = "06c14271ee966e108d55ae109f340976a9556c8603e888037145d6522726aebe89dd0c861b4b83947feaf6d39e79e08817559e8693deedc2c94e82c5cbd090c7"; };
"domhandler@npm:4.3.0" = { filename = "domhandler-npm-4.3.0-d142f8b6f6-d2a2dbf40d.zip"; sha512 = "d2a2dbf40dd99abf936b65ad83c6b530afdb3605a87cad37a11b5d9220e68423ebef1b86c89e0f6d93ffaf315cc327cf1a988652e7a9a95cce539e3984f4c64d"; };
"koa-router@npm:7.4.0" = { filename = "koa-router-npm-7.4.0-0a220ac73d-fa39643a00.zip"; sha512 = "fa39643a00d12964e0c0479beb9987b58b4a68367b0f2e93a5b9b7187912442ca50ceda31c084b49407d77ea4d9ba1fbcc432170cc528f5774dee738df9dcb4e"; };
"telegraf@npm:3.40.0" = { filename = "telegraf-npm-3.40.0-11c701c123-59cef0a275.zip"; sha512 = "59cef0a275685189d632f6843080f6b3e20d9d13def0b3c673b6000a52ca407165b6c2074ec0dacf5e270c64142d830da03b90f27de3a7671e69735801cc018d"; };
"is-arrayish@npm:0.3.2" = { filename = "is-arrayish-npm-0.3.2-f856180f79-977e64f54d.zip"; sha512 = "977e64f54d91c8f169b59afcd80ff19227e9f5c791fa28fa2e5bce355cbaf6c2c356711b734656e80c9dd4a854dd7efcf7894402f1031dfc5de5d620775b4d5f"; };
"fs-constants@npm:1.0.0" = { filename = "fs-constants-npm-1.0.0-59576b2177-18f5b71837.zip"; sha512 = "18f5b718371816155849475ac36c7d0b24d39a11d91348cfcb308b4494824413e03572c403c86d3a260e049465518c4f0d5bd00f0371cdfcad6d4f30a85b350d"; };
"buffer@npm:5.7.1" = { filename = "buffer-npm-5.7.1-513ef8259e-e2cf8429e1.zip"; sha512 = "e2cf8429e1c4c7b8cbd30834ac09bd61da46ce35f5c22a78e6c2f04497d6d25541b16881e30a019c6fd3154150650ccee27a308eff3e26229d788bbdeb08ab84"; };
"node-abi@npm:3.8.0" = { filename = "node-abi-npm-3.8.0-bba572a0b8-3644dd51f4.zip"; sha512 = "3644dd51f4f189358ef56055407501aa698632d67448585b38c46c81a482a0c3bfb06da513ac4060a12ce5f607f208ba9d9c8280f1c38329670b709bd735fcae"; };
"yaeti@npm:0.0.6" = { filename = "yaeti-npm-0.0.6-cffd01e35a-6db12c152f.zip"; sha512 = "6db12c152f7c363b80071086a3ebf5032e03332604eeda988872be50d6c8469e1f13316175544fa320f72edad696c2d83843ad0ff370659045c1a68bcecfcfea"; };
"are-we-there-yet@npm:3.0.0" = { filename = "are-we-there-yet-npm-3.0.0-1391430190-348edfdd93.zip"; sha512 = "348edfdd931b0b50868b55402c01c3f64df1d4c229ab6f063539a5025fd6c5f5bb8a0cab409bbed8d75d34762d22aa91b7c20b4204eb8177063158d9ba792981"; };
"chownr@npm:2.0.0" = { filename = "chownr-npm-2.0.0-638f1c9c61-c57cf9dd07.zip"; sha512 = "c57cf9dd0791e2f18a5ee9c1a299ae6e801ff58fee96dc8bfd0dcb4738a6ce58dd252a3605b1c93c6418fe4f9d5093b28ffbf4d66648cb2a9c67eaef9679be2f"; };
"mkdirp@npm:1.0.4" = { filename = "mkdirp-npm-1.0.4-37f6ef56b9-a96865108c.zip"; sha512 = "a96865108c6c3b1b8e1d5e9f11843de1e077e57737602de1b82030815f311be11f96f09cce59bd5b903d0b29834733e5313f9301e3ed6d6f6fba2eae0df4298f"; };
"nan@npm:2.17.0" = { filename = "nan-npm-2.17.0-bf36a21d6f-ec609aeaf7.zip"; sha512 = "ec609aeaf7e68b76592a3ba96b372aa7f5df5b056c1e37410b0f1deefbab5a57a922061e2c5b369bae9c7c6b5e6eecf4ad2dac8833a1a7d3a751e0a7c7f849ed"; };
"bytes@npm:3.1.2" = { filename = "bytes-npm-3.1.2-28b8643004-e4bcd3948d.zip"; sha512 = "e4bcd3948d289c5127591fbedf10c0b639ccbf00243504e4e127374a15c3bc8eed0d28d4aaab08ff6f1cf2abc0cce6ba3085ed32f4f90e82a5683ce0014e1b6e"; };
"humanize-number@npm:0.0.2" = { filename = "humanize-number-npm-0.0.2-ac26ad7632-9c98c9d06b.zip"; sha512 = "9c98c9d06b0f3d801960be3957199232a5df52377e2502acae92e4f71de633fa62c315a83f24bf96bef76f47b2e3e0e1e4f4157c891e27074fd3272cad6724bb"; };
"passthrough-counter@npm:1.0.0" = { filename = "passthrough-counter-npm-1.0.0-cbb2a89ced-942a0addeb.zip"; sha512 = "942a0addeb677e24ddb154b04cc29ce1c5720032efc268689446420f9350d47e94f2f1f76d469686bc87c1543c2f2165f2d004d265fe1b81465c76e02d272c63"; };
"lodash.defaults@npm:4.2.0" = { filename = "lodash.defaults-npm-4.2.0-c5dea025ab-8492325823.zip"; sha512 = "84923258235592c8886e29de5491946ff8c2ae5c82a7ac5cddd2e3cb697e6fbdfbbb6efcca015795c86eec2bb953a5a2ee4016e3735a3f02720428a40efbb8f1"; };
"ratelimiter@npm:3.4.1" = { filename = "ratelimiter-npm-3.4.1-5ba5f26b13-e445142387.zip"; sha512 = "e4451423871405aafa300399063d3a0aefef3eb42704d09cf95a0d3f70bdd74865397e408cd78aeb695cbf7afc7e5a34481e412bd2a04eeabdf7c52a48bae87e"; };
"domutils@npm:2.8.0" = { filename = "domutils-npm-2.8.0-0325139e5c-abf7434315.zip"; sha512 = "abf7434315283e9aadc2a24bac0e00eab07ae4313b40cc239f89d84d7315ebdfd2fb1b5bf750a96bc1b4403d7237c7b2ebf60459be394d625ead4ca89b934391"; };
"typedarray-to-buffer@npm:3.1.5" = { filename = "typedarray-to-buffer-npm-3.1.5-aadc11995e-99c11aaa8f.zip"; sha512 = "99c11aaa8f45189fcfba6b8a4825fd684a321caa9bd7a76a27cf0c7732c174d198b99f449c52c3818107430b5f41c0ccbbfb75cb2ee3ca4a9451710986d61a60"; };
"brace-expansion@npm:1.1.11" = { filename = "brace-expansion-npm-1.1.11-fb95eb05ad-faf34a7bb0.zip"; sha512 = "faf34a7bb0c3fcf4b59c7808bc5d2a96a40988addf2e7e09dfbb67a2251800e0d14cd2bfc1aa79174f2f5095c54ff27f46fb1289fe2d77dac755b5eb3434cc07"; };
"universalify@npm:2.0.0" = { filename = "universalify-npm-2.0.0-03b8b418a8-2406a4edf4.zip"; sha512 = "2406a4edf4a8830aa6813278bab1f953a8e40f2f63a37873ffa9a3bc8f9745d06cc8e88f3572cb899b7e509013f7f6fcc3e37e8a6d914167a5381d8440518c44"; };
"fs-minipass@npm:2.1.0" = { filename = "fs-minipass-npm-2.1.0-501ef87306-1b8d128dae.zip"; sha512 = "1b8d128dae2ac6cc94230cc5ead341ba3e0efaef82dab46a33d171c044caaa6ca001364178d42069b2809c35a1c3c35079a32107c770e9ffab3901b59af8c8b1"; };
"minipass@npm:3.1.6" = { filename = "minipass-npm-3.1.6-f032df1661-57a0404141.zip"; sha512 = "57a04041413a3531a65062452cb5175f93383ef245d6f4a2961d34386eb9aa8ac11ac7f16f791f5e8bbaf1dfb1ef01596870c88e8822215db57aa591a5bb0a77"; };
"imurmurhash@npm:0.1.4" = { filename = "imurmurhash-npm-0.1.4-610c5068a0-7cae75c8cd.zip"; sha512 = "7cae75c8cd9a50f57dadd77482359f659eaebac0319dd9368bcd1714f55e65badd6929ca58569da2b6494ef13fdd5598cd700b1eba23f8b79c5f19d195a3ecf7"; };
"slide@npm:1.1.6" = { filename = "slide-npm-1.1.6-14e4650dea-5768635d22.zip"; sha512 = "5768635d227172e215b7a1a91d32f8781f5783b4961feaaf3d536bbf83cc51878928c137508cde7659fea6d7c04074927cab982731302771ee0051518ff24896"; };
"regenerator-runtime@npm:0.13.11" = { filename = "regenerator-runtime-npm-0.13.11-90bf536060-27481628d2.zip"; sha512 = "27481628d22a1c4e3ff551096a683b424242a216fee44685467307f14d58020af1e19660bf2e26064de946bad7eff28950eae9f8209d55723e2d9351e632bbb4"; };
"sharp@npm:0.30.7" = { filename = "sharp-npm-0.30.7-6db6fef43e-bbc63ca3c7.zip"; sha512 = "bbc63ca3c7ea8a5bff32cd77022cfea30e25a03f5bd031e935924bf6cf0e11e3388e8b0e22b3137bf8816aa73407f1e4fbeb190f3a35605c27ffca9f32b91601"; };
"bl@npm:4.1.0" = { filename = "bl-npm-4.1.0-7f94cdcf3f-9e8521fa7e.zip"; sha512 = "9e8521fa7e83aa9427c6f8ccdcba6e8167ef30cc9a22df26effcc5ab682ef91d2cbc23a239f945d099289e4bbcfae7a192e9c28c84c6202e710a0dfec3722662"; };
"minizlib@npm:2.1.2" = { filename = "minizlib-npm-2.1.2-ea89cd0cfb-f1fdeac0b0.zip"; sha512 = "f1fdeac0b07cf8f30fcf12f4b586795b97be856edea22b5e9072707be51fc95d41487faec3f265b42973a304fe3a64acd91a44a3826a963e37b37bafde0212c3"; };
"simple-get@npm:3.1.1" = { filename = "simple-get-npm-3.1.1-dce5923dba-80195e70bf.zip"; sha512 = "80195e70bf171486e75c31e28e5485468195cc42f85940f8b45c4a68472160144d223eb4d07bc82ef80cb974b7c401db021a540deb2d34ac4b3b8883da2d6401"; };
"@babel/runtime@npm:7.20.7" = { filename = "@babel-runtime-npm-7.20.7-69d8df458c-4629ce5c46.zip"; sha512 = "4629ce5c46f06cca9cfb9b7fc00d48003335a809888e2b91ec2069a2dcfbfef738480cff32ba81e0b7c290f8918e5c22ddcf2b710001464ee84ba62c7e32a3a3"; };
"co-body@npm:6.1.0" = { filename = "co-body-npm-6.1.0-8102b96671-d0a78831a6.zip"; sha512 = "d0a78831a6651f2085fce16b0ecdc49f45fb5baf4f94148c2f499e7ec89d188205362548b9c500eae15a819360cfda208079e68a72c204cf66ca3ffa2fc0f57e"; };
"methods@npm:1.1.2" = { filename = "methods-npm-1.1.2-92f6fdb39b-0917ff4041.zip"; sha512 = "0917ff4041fa8e2f2fda5425a955fe16ca411591fbd123c0d722fcf02b73971ed6f764d85f0a6f547ce49ee0221ce2c19a5fa692157931cecb422984f1dcd13a"; };
"urijs@npm:1.19.11" = { filename = "urijs-npm-1.19.11-e9c694fd1d-f9b9500456.zip"; sha512 = "f9b95004560754d30fd7dbee44b47414d662dc9863f1cf5632a7c7983648df11d23c0be73b9b4f9554463b61d5b0a520b70df9e1ee963ebb4af02e6da2cc80f3"; };
"module-alias@npm:2.2.2" = { filename = "module-alias-npm-2.2.2-576bd37c03-4b5543f834.zip"; sha512 = "4b5543f834b484033e5bd184096ca8276b9195e32e88883ee6ea8d3a4789d97c470d26f5fa7271bd7a26588bf67e4d27dbdb594ee327aef1c9619d855dc78342"; };
"sandwich-stream@npm:2.0.2" = { filename = "sandwich-stream-npm-2.0.2-76bdc6c560-666d3276e5.zip"; sha512 = "666d3276e5390786a4ad69f04da472fb69940d1a279a22aebd49ae78ca8f12855503e69d71f77d4b415144a284ffa14efa811a097ad7df501a68c01438b91282"; };
"typegram@npm:3.12.0" = { filename = "typegram-npm-3.12.0-75cc7a1e63-30f1579777.zip"; sha512 = "30f1579777a8c84f62d65b335490e1186e56179d37d622baa00d7be57bbbcb2b4d1dd9d1a6a081f012ef56c2fb9cdf8098733dbce4b2d98f824d109a5b2dc846"; };
"rc@npm:1.2.8" = { filename = "rc-npm-1.2.8-d6768ac936-2e26e052f8.zip"; sha512 = "2e26e052f8be2abd64e6d1dabfbd7be03f80ec18ccbc49562d31f617d0015fbdbcf0f9eed30346ea6ab789e0fdfe4337f033f8016efdbee0df5354751842080e"; };
"es5-ext@npm:0.10.53" = { filename = "es5-ext-npm-0.10.53-18c0039c41-24ec223692.zip"; sha512 = "24ec22369260cf98605cb2f51eae9d7df5dc621bc5d3b311f6f5c3d0fcdb7bafae888270f3083ee6e9af27350a5ea49f1fe2dd6406a9017247ca40f091f529b2"; };
"jsonfile@npm:6.1.0" = { filename = "jsonfile-npm-6.1.0-20a4796cee-7af3b8e1ac.zip"; sha512 = "7af3b8e1ac8fe7f1eccc6263c6ca14e1966fcbc74b618d3c78a0a2075579487547b94f72b7a1114e844a1e15bb00d440e5d1720bfc4612d790a6f285d5ea8354"; };
"@jimp/custom@npm:0.16.2" = { filename = "@jimp-custom-npm-0.16.2-e0de91039c-2edca54cc8.zip"; sha512 = "2edca54cc87144e7668b1e154150abfe6663baa83d609035ae5f2412cc767345c33c0b7700b7b1a4995e1d975301712568f1b548f55a6dc03634bd4bfd4492e9"; };
"chalk@npm:2.4.2" = { filename = "chalk-npm-2.4.2-3ea16dd91e-ec3661d38f.zip"; sha512 = "ec3661d38fe77f681200f878edbd9448821924e0f93a9cefc0e26a33b145f1027a2084bf19967160d11e1f03bfe4eaffcabf5493b89098b2782c3fe0b03d80c2"; };
"koa-compose@npm:3.2.1" = { filename = "koa-compose-npm-3.2.1-0020b997fa-ff8e5fc034.zip"; sha512 = "ff8e5fc0348455acf751179c6c613eb030a5fac6406d3b49ae9e00460b7ee8770db3ef62633fd3db0306cd4a6d2a0b5152399ebd5bb5e684418f9eeeb251c2de"; };
"path-to-regexp@npm:1.8.0" = { filename = "path-to-regexp-npm-1.8.0-a1904f5c44-709f6f083c.zip"; sha512 = "709f6f083c0552514ef4780cb2e7e4cf49b0cc89a97439f2b7cc69a608982b7690fb5d1720a7473a59806508fc2dae0be751ba49f495ecf89fd8fbc62abccbcd"; };
"node-fetch@npm:2.6.8" = { filename = "node-fetch-npm-2.6.8-ddb8bfc274-91f57be68e.zip"; sha512 = "91f57be68e29f9b1382750693619e199733a6936998e6d618f1aa779853ad8fc4a2facf170db7957bf1d2510bad33449edf74b5802713d81b63de5986fa3be00"; };
"delegates@npm:1.0.0" = { filename = "delegates-npm-1.0.0-9b1942d75f-a51744d9b5.zip"; sha512 = "a51744d9b53c164ba9c0492471a1a2ffa0b6727451bdc89e31627fdf4adda9d51277cfcbfb20f0a6f08ccb3c436f341df3e92631a3440226d93a8971724771fd"; };
"is-typedarray@npm:1.0.0" = { filename = "is-typedarray-npm-1.0.0-bbd99de5b6-3508c6cd0a.zip"; sha512 = "3508c6cd0a9ee2e0df2fa2e9baabcdc89e911c7bd5cf64604586697212feec525aa21050e48affb5ffc3df20f0f5d2e2cf79b08caa64e1ccc9578e251763aef7"; };
"balanced-match@npm:1.0.2" = { filename = "balanced-match-npm-1.0.2-a53c126459-9706c088a2.zip"; sha512 = "9706c088a283058a8a99e0bf91b0a2f75497f185980d9ffa8b304de1d9e58ebda7c72c07ebf01dadedaac5b2907b2c6f566f660d62bd336c3468e960403b9d65"; };
"concat-map@npm:0.0.1" = { filename = "concat-map-npm-0.0.1-85a921b7ee-902a9f5d89.zip"; sha512 = "902a9f5d8967a3e2faf138d5cb784b9979bad2e6db5357c5b21c568df4ebe62bcb15108af1b2253744844eb964fc023fbd9afbbbb6ddd0bcc204c6fb5b7bf3af"; };
"inflation@npm:2.0.0" = { filename = "inflation-npm-2.0.0-e638c91672-a0494871b1.zip"; sha512 = "a0494871b12275afdef9e2710ee1af1e0fc642b04613a9be69c05ef8b5e9627f3bd7d358a937fa47aa20235ee7313a4f30255048533add0ad4918beb918a586e"; };
"deep-extend@npm:0.6.0" = { filename = "deep-extend-npm-0.6.0-e182924219-7be7e5a8d4.zip"; sha512 = "7be7e5a8d468d6b10e6a67c3de828f55001b6eb515d014f7aeb9066ce36bd5717161eb47d6a0f7bed8a9083935b465bc163ee2581c8b128d29bf61092fdf57a7"; };
"ini@npm:1.3.8" = { filename = "ini-npm-1.3.8-fb5040b4c0-dfd98b0ca3.zip"; sha512 = "dfd98b0ca3a4fc1e323e38a6c8eb8936e31a97a918d3b377649ea15bdb15d481207a0dda1021efbd86b464cae29a0d33c1d7dcaf6c5672bee17fa849bc50a1b3"; };
"strip-json-comments@npm:2.0.1" = { filename = "strip-json-comments-npm-2.0.1-e7883b2d04-1074ccb632.zip"; sha512 = "1074ccb63270d32ca28edfb0a281c96b94dc679077828135141f27d52a5a398ef5e78bcf22809d23cadc2b81dfbe345eb5fd8699b385c8b1128907dec4a7d1e1"; };
"next-tick@npm:1.0.0" = { filename = "next-tick-npm-1.0.0-0c0dd4bec1-83fcb3d4f8.zip"; sha512 = "83fcb3d4f8d9380210b1c2b8a610463602d80283f0c0c8571c1688e1ad6cbf3a16b345f5bb7212617d4898bedcfa10dff327dc09ec20a112a5bf43a0271375fb"; };
"make-fetch-happen@npm:9.1.0" = { filename = "make-fetch-happen-npm-9.1.0-23184ad7f6-0eb371c85f.zip"; sha512 = "0eb371c85fdd0b1584fcfdf3dc3c62395761b3c14658be02620c310305a9a7ecf1617a5e6fb30c1d081c5c8aaf177fa133ee225024313afabb7aa6a10f1e3d04"; };
"decompress-response@npm:4.2.1" = { filename = "decompress-response-npm-4.2.1-abe5b4ebe4-4e783ca4df.zip"; sha512 = "4e783ca4dfe9417354d61349750fe05236f565a4415a6ca20983a311be2371debaedd9104c0b0e7b36e5f167aeaae04f84f1a0b3f8be4162f1d7d15598b8fdba"; };
"qs@npm:6.11.0" = { filename = "qs-npm-6.11.0-caf1bc9dea-6e1f29dd53.zip"; sha512 = "6e1f29dd5385f7488ec74ac7b6c92f4d09a90408882d0c208414a34dd33badc1a621019d4c799a3df15ab9b1d0292f97c1dd71dc7c045e69f81a8064e5af7297"; };
"escape-string-regexp@npm:1.0.5" = { filename = "escape-string-regexp-npm-1.0.5-3284de402f-6092fda75c.zip"; sha512 = "6092fda75c63b110c706b6a9bfde8a612ad595b628f0bd2147eea1d3406723020810e591effc7db1da91d80a71a737a313567c5abb3813e8d9c71f4aa595b410"; };
"any-promise@npm:1.3.0" = { filename = "any-promise-npm-1.3.0-f34eeaa7e7-0ee8a9bdbe.zip"; sha512 = "0ee8a9bdbe882c90464d75d1f55cf027f5458650c4bd1f0467e65aec38ccccda07ca5844969ee77ed46d04e7dded3eaceb027e8d32f385688523fe305fa7e1de"; };
"isarray@npm:0.0.1" = { filename = "isarray-npm-0.0.1-92e37e0a70-49191f1425.zip"; sha512 = "49191f1425681df4a18c2f0f93db3adb85573bcdd6a4482539d98eac9e705d8961317b01175627e860516a2fc45f8f9302db26e5a380a97a520e272e2a40a8d4"; };
"gauge@npm:4.0.0" = { filename = "gauge-npm-4.0.0-962bc1259b-637b34c84f.zip"; sha512 = "637b34c84f518defa89319dbef68211a24e9302182ad2a619e3be1be5b7dcf2a962c8359e889294af667440f4722e7e6e61671859e00bd8ec280a136ded89b25"; };
"dom-serializer@npm:1.3.2" = { filename = "dom-serializer-npm-1.3.2-133de2b9ce-bff4871494.zip"; sha512 = "bff48714944d67b160db71ba244fb0f3fe72e77ef2ec8414e2eeb56f2d926e404a13456b8b83a5392e217ba47dec2ec0c368801b31481813e94d185276c3e964"; };
"http-errors@npm:1.8.1" = { filename = "http-errors-npm-1.8.1-fb60d9f6ae-d3c7e7e776.zip"; sha512 = "d3c7e7e776fd51c0a812baff570bdf06fe49a5dc448b700ab6171b1250e4cf7db8b8f4c0b133e4bfe2451022a5790c1ca6c2cae4094dedd6ac8304a1267f91d2"; };
"type-is@npm:1.6.18" = { filename = "type-is-npm-1.6.18-6dee4d4961-2c8e47675d.zip"; sha512 = "2c8e47675d55f8b4e404bcf529abdf5036c537a04c2b20177bcf78c9e3c1da69da3942b1346e6edb09e823228c0ee656ef0e033765ec39a70d496ef601a0c657"; };
"ansi-styles@npm:3.2.1" = { filename = "ansi-styles-npm-3.2.1-8cb8107983-d85ade01c1.zip"; sha512 = "d85ade01c10e5dd77b6c89f34ed7531da5830d2cb5882c645f330079975b716438cd7ebb81d0d6e6b4f9c577f19ae41ab55f07f19786b02f9dfd9e0377395665"; };
"supports-color@npm:5.5.0" = { filename = "supports-color-npm-5.5.0-183ac537bc-95f6f4ba5a.zip"; sha512 = "95f6f4ba5afdf92f495b5a912d4abee8dcba766ae719b975c56c084f5004845f6f5a5f7769f52d53f40e21952a6d87411bafe34af4a01e65f9926002e38e1dac"; };
"es6-symbol@npm:3.1.3" = { filename = "es6-symbol-npm-3.1.3-34d72f2a23-cd49722c2a.zip"; sha512 = "cd49722c2a70f011eb02143ef1c8c70658d2660dead6641e160b94619f408b9cf66425515787ffe338affdf0285ad54f4eae30ea5bd510e33f8659ec53bcaa70"; };
"@mapbox/node-pre-gyp@npm:1.0.10" = { filename = "@mapbox-node-pre-gyp-npm-1.0.10-1811160047-1a98db05d9.zip"; sha512 = "1a98db05d955b74dad3814679593df293b9194853698f3f5f1ed00ecd93128cdd4b14fb8767fe44ac6981ef05c23effcfdc88710e7c1de99ccb6f647890597c8"; };
"@jimp/types@npm:0.16.2" = { filename = "@jimp-types-npm-0.16.2-57bff72cff-529e234a4c.zip"; sha512 = "529e234a4c8ee3470dbedeef8c41bde702e5aebb4049c0b7b66fda6972039d74618236fd36e316b7393670106264011ae89e55b370b9589a6a4b6cddcca26eff"; };
"whatwg-url@npm:5.0.0" = { filename = "whatwg-url-npm-5.0.0-374fb45e60-b8daed4ad3.zip"; sha512 = "b8daed4ad3356cc4899048a15b2c143a9aed0dfae1f611ebd55073310c7b910f522ad75d727346ad64203d7e6c79ef25eafd465f4d12775ca44b90fa82ed9e2c"; };
"es6-iterator@npm:2.0.3" = { filename = "es6-iterator-npm-2.0.3-4dadb0ccc1-6e48b1c2d9.zip"; sha512 = "6e48b1c2d962c21dee604b3d9f0bc3889f11ed5a8b33689155a2065d20e3107e2a69cc63a71bd125aeee3a589182f8bbcb5c8a05b6a8f38fa4205671b6d09697"; };
"raw-body@npm:2.5.1" = { filename = "raw-body-npm-2.5.1-9dd1d9fff9-5362adff15.zip"; sha512 = "5362adff1575d691bb3f75998803a0ffed8c64eabeaa06e54b4ada25a0cd1b2ae7f4f5ec46565d1bec337e08b5ac90c76eaa0758de6f72a633f025d754dec29e"; };
"http-cache-semantics@npm:4.1.0" = { filename = "http-cache-semantics-npm-4.1.0-860520a31f-974de94a81.zip"; sha512 = "974de94a81c5474be07f269f9fd8383e92ebb5a448208223bfb39e172a9dbc26feff250192ecc23b9593b3f92098e010406b0f24bd4d588d631f80214648ed42"; };
"is-lambda@npm:1.0.1" = { filename = "is-lambda-npm-1.0.1-7ab55bc8a8-93a32f0194.zip"; sha512 = "93a32f01940220532e5948538699ad610d5924ac86093fcee83022252b363eb0cc99ba53ab084a04e4fb62bf7b5731f55496257a4c38adf87af9c4d352c71c35"; };
"negotiator@npm:0.6.3" = { filename = "negotiator-npm-0.6.3-9d50e36171-b8ffeb1e26.zip"; sha512 = "b8ffeb1e262eff7968fc90a2b6767b04cfd9842582a9d0ece0af7049537266e7b2506dfb1d107a32f06dd849ab2aea834d5830f7f4d0e5cb7d36e1ae55d021d9"; };
"mimic-response@npm:2.1.0" = { filename = "mimic-response-npm-2.1.0-037463e454-014fad6ab9.zip"; sha512 = "014fad6ab936657e5f2f48bd87af62a8e928ebe84472aaf9e14fec4fcb31257a5edff77324d8ac13ddc6685ba5135cf16e381efac324e5f174fb4ddbf902bf07"; };
"aproba@npm:2.0.0" = { filename = "aproba-npm-2.0.0-8716bcfde6-5615cadcfb.zip"; sha512 = "5615cadcfb45289eea63f8afd064ab656006361020e1735112e346593856f87435e02d8dcc7ff0d11928bc7d425f27bc7c2a84f6c0b35ab0ff659c814c138a24"; };
"color-support@npm:1.1.3" = { filename = "color-support-npm-1.1.3-3be5c53455-9b73568176.zip"; sha512 = "9b7356817670b9a13a26ca5af1c21615463b500783b739b7634a0c2047c16cef4b2865d7576875c31c3cddf9dd621fa19285e628f20198b233a5cfdda6d0793b"; };
"has-unicode@npm:2.0.1" = { filename = "has-unicode-npm-2.0.1-893adb4747-1eab07a743.zip"; sha512 = "1eab07a7436512db0be40a710b29b5dc21fa04880b7f63c9980b706683127e3c1b57cb80ea96d47991bdae2dfe479604f6a1ba410106ee1046a41d1bd0814400"; };
"signal-exit@npm:3.0.7" = { filename = "signal-exit-npm-3.0.7-bd270458a3-a2f098f247.zip"; sha512 = "a2f098f247adc367dffc27845853e9959b9e88b01cb301658cfe4194352d8d2bb32e18467c786a7fe15f1d44b233ea35633d076d5e737870b7139949d1ab6318"; };
"depd@npm:1.1.2" = { filename = "depd-npm-1.1.2-b0c8414da7-6b406620d2.zip"; sha512 = "6b406620d269619852885ce15965272b829df6f409724415e0002c8632ab6a8c0a08ec1f0bd2add05dc7bd7507606f7e2cc034fa24224ab829580040b835ecd9"; };
"setprototypeof@npm:1.2.0" = { filename = "setprototypeof-npm-1.2.0-0fedbdcd3a-be18cbbf70.zip"; sha512 = "be18cbbf70e7d8097c97f713a2e76edf84e87299b40d085c6bf8b65314e994cc15e2e317727342fa6996e38e1f52c59720b53fe621e2eb593a6847bf0356db89"; };
"statuses@npm:1.5.0" = { filename = "statuses-npm-1.5.0-f88f91b2e9-c469b9519d.zip"; sha512 = "c469b9519de16a4bb19600205cffb39ee471a5f17b82589757ca7bd40a8d92ebb6ed9f98b5a540c5d302ccbc78f15dc03cc0280dd6e00df1335568a5d5758a5c"; };
"toidentifier@npm:1.0.1" = { filename = "toidentifier-npm-1.0.1-f759712599-952c29e2a8.zip"; sha512 = "952c29e2a85d7123239b5cfdd889a0dde47ab0497f0913d70588f19c53f7e0b5327c95f4651e413c74b785147f9637b17410ac8c846d5d4a20a5a33eb6dc3a45"; };
"minipass-collect@npm:1.0.2" = { filename = "minipass-collect-npm-1.0.2-3b4676eab5-14df761028.zip"; sha512 = "14df761028f3e47293aee72888f2657695ec66bd7d09cae7ad558da30415fdc4752bbfee66287dcc6fd5e6a2fa3466d6c484dc1cbd986525d9393b9523d97f10"; };
"minipass-flush@npm:1.0.5" = { filename = "minipass-flush-npm-1.0.5-efe79d9826-56269a0b22.zip"; sha512 = "56269a0b22bad756a08a94b1ffc36b7c9c5de0735a4dd1ab2b06c066d795cfd1f0ac44a0fcae13eece5589b908ecddc867f04c745c7009be0b566421ea0944cf"; };
"minipass-pipeline@npm:1.2.4" = { filename = "minipass-pipeline-npm-1.2.4-5924cb077f-b14240dac0.zip"; sha512 = "b14240dac0d29823c3d5911c286069e36d0b81173d7bdf07a7e4a91ecdef92cdff4baaf31ea3746f1c61e0957f652e641223970870e2353593f382112257971b"; };
"ssri@npm:8.0.1" = { filename = "ssri-npm-8.0.1-a369e72ce2-bc447f5af8.zip"; sha512 = "bc447f5af814fa9713aa201ec2522208ae0f4d8f3bda7a1f445a797c7b929a02720436ff7c478fb5edc4045adb02b1b88d2341b436a80798734e2494f1067b36"; };
"media-typer@npm:0.3.0" = { filename = "media-typer-npm-0.3.0-8674f8f0f5-af1b38516c.zip"; sha512 = "af1b38516c28ec95d6b0826f6c8f276c58aec391f76be42aa07646b4e39d317723e869700933ca6995b056db4b09a78c92d5440dc23657e6764be5d28874bba1"; };
"has-flag@npm:3.0.0" = { filename = "has-flag-npm-3.0.0-16ac11fe05-4a15638b45.zip"; sha512 = "4a15638b454bf086c8148979aae044dd6e39d63904cd452d970374fa6a87623423da485dfb814e7be882e05c096a7ccf1ebd48e7e7501d0208d8384ff4dea73b"; };
"wide-align@npm:1.1.5" = { filename = "wide-align-npm-1.1.5-889d77e592-d5fc37cd56.zip"; sha512 = "d5fc37cd561f9daee3c80e03b92ed3e84d80dde3365a8767263d03dacfc8fa06b065ffe1df00d8c2a09f731482fcacae745abfbb478d4af36d0a891fad4834d3"; };
"timm@npm:1.7.1" = { filename = "timm-npm-1.7.1-4d0984663a-c80df538ec.zip"; sha512 = "c80df538ec7fae50a0e3183931b20fbe97f6f2c06907d9675eb7b9d90b3f788af7742285c730192db3b066c4ab22ebae75f8d21970c5b03f38d928d5bb2a0339"; };
"https-proxy-agent@npm:5.0.0" = { filename = "https-proxy-agent-npm-5.0.0-bb777903c3-165bfb090b.zip"; sha512 = "165bfb090bd26d47693597661298006841ab733d0c7383a8cb2f17373387a94c903a3ac687090aa739de05e379ab6f868bae84ab4eac288ad85c328cd1ec9e53"; };
"promise-retry@npm:2.0.1" = { filename = "promise-retry-npm-2.0.1-871f0b01b7-f96a3f6d90.zip"; sha512 = "f96a3f6d90b92b568a26f71e966cbbc0f63ab85ea6ff6c81284dc869b41510e6cdef99b6b65f9030f0db422bf7c96652a3fff9f2e8fb4a0f069d8f4430359429"; };
"color-convert@npm:1.9.3" = { filename = "color-convert-npm-1.9.3-1fe690075e-fd7a64a17c.zip"; sha512 = "fd7a64a17cde98fb923b1dd05c5f2e6f7aefda1b60d67e8d449f9328b4e53b228a428fd38bfeaeb2db2ff6b6503a776a996150b80cdf224062af08a5c8a3a203"; };
"tr46@npm:0.0.3" = { filename = "tr46-npm-0.0.3-de53018915-726321c5ea.zip"; sha512 = "726321c5eaf41b5002e17ffbd1fb7245999a073e8979085dacd47c4b4e8068ff5777142fc6726d6ca1fd2ff16921b48788b87225cbc57c72636f6efa8efbffe3"; };
"webidl-conversions@npm:3.0.1" = { filename = "webidl-conversions-npm-3.0.1-60310f6a2b-c92a0a6ab9.zip"; sha512 = "c92a0a6ab95314bde9c32e1d0a6dfac83b578f8fa5f21e675bc2706ed6981bc26b7eb7e6a1fab158e5ce4adf9caa4a0aee49a52505d4d13c7be545f15021b17c"; };
"ext@npm:1.6.0" = { filename = "ext-npm-1.6.0-d724ecc103-ca3ef4619e.zip"; sha512 = "ca3ef4619e838f441a92238a98b77ac873da2175ace746c64303ffe2c3208e79a3acf3bf7004e40b720f3c2a83bf0143e6dd4a7cdfae6e73f54a3bfc7a14b5c2"; };
"make-dir@npm:3.1.0" = { filename = "make-dir-npm-3.1.0-d1d7505142-484200020a.zip"; sha512 = "484200020ab5a1fdf12f393fe5f385fc8e4378824c940fba1729dcd198ae4ff24867bc7a5646331e50cead8abff5d9270c456314386e629acec6dff4b8016b78"; };
"agentkeepalive@npm:4.2.0" = { filename = "agentkeepalive-npm-4.2.0-e5e72b8ce4-89806f83ce.zip"; sha512 = "89806f83ceebbcaabf6bd581a8dce4870910fd2a11f66df8f505b4cd4ce4ca5ab9e6eec8d11ce8531a6b60f6748b75b0775e0e2fa33871503ef00d535418a19a"; };
"http-proxy-agent@npm:4.0.1" = { filename = "http-proxy-agent-npm-4.0.1-ce9ef61788-c6a5da5a19.zip"; sha512 = "c6a5da5a1929416b6bbdf77b1aca13888013fe7eb9d59fc292e25d18e041bb154a8dfada58e223fc7b76b9b2d155a87e92e608235201f77d34aa258707963a82"; };
"socks-proxy-agent@npm:6.1.1" = { filename = "socks-proxy-agent-npm-6.1.1-a3843946ba-9a8a4f791b.zip"; sha512 = "9a8a4f791bba0060315cf7291ca6f9db37d6fc280fd0860d73d8887d3efe4c22e823aa25a8d5375f6079279f8dc91b50c075345179bf832bfe3c7c26d3582e3c"; };
"side-channel@npm:1.0.4" = { filename = "side-channel-npm-1.0.4-e1f38b9e06-351e41b947.zip"; sha512 = "351e41b947079c10bd0858364f32bb3a7379514c399edb64ab3dce683933483fc63fb5e4efe0a15a2e8a7e3c436b6a91736ddb8d8c6591b0460a24bb4a1ee245"; };
"unpipe@npm:1.0.0" = { filename = "unpipe-npm-1.0.0-2ed2a3c2bf-4fa18d8d8d.zip"; sha512 = "4fa18d8d8d977c55cb09715385c203197105e10a6d220087ec819f50cb68870f02942244f1017565484237f1f8c5d3cd413631b1ae104d3096f24fdfde1b4aa2"; };
"d@npm:1.0.1" = { filename = "d-npm-1.0.1-64afbbc689-49ca0639c7.zip"; sha512 = "49ca0639c7b822db670de93d4fbce44b4aa072cd848c76292c9978a8cd0fff1028763020ff4b0f147bd77bfe29b4c7f82e0f71ade76b2a06100543cdfd948d19"; };
"@jimp/tiff@npm:0.16.2" = { filename = "@jimp-tiff-npm-0.16.2-092d9857c1-7fc11c13c1.zip"; sha512 = "7fc11c13c1ca8e65659b798af4ef8d06a2e2bce825805c1b448ccc9376999a7da051d33c64497c44d75dfc6127cd3d15753716c9e8e6678acf8efdf54b9d088a"; };
"minipass-fetch@npm:1.4.1" = { filename = "minipass-fetch-npm-1.4.1-2d67357feb-ec93697bdb.zip"; sha512 = "ec93697bdb62129c4e6c0104138e681e30efef8c15d9429dd172f776f83898471bc76521b539ff913248cc2aa6d2b37b652c993504a51cc53282563640f29216"; };
"@jimp/bmp@npm:0.16.2" = { filename = "@jimp-bmp-npm-0.16.2-dfc8dc8414-e52d2f77d5.zip"; sha512 = "e52d2f77d5d32678f0245b7a77fa0a4d52b37ec198b9605e78c0c3de63b1cc3c7ef06fefc2732d675edac9e941ca4c95bbced81ff5a698f2073e8ec2e2258f4c"; };
"@jimp/jpeg@npm:0.16.2" = { filename = "@jimp-jpeg-npm-0.16.2-4b53f48593-cae9d8fc32.zip"; sha512 = "cae9d8fc323d2d673b0104aebc60764de9681c5c213417b8cc67d11d1251361a7f5b0bc37fb76fc84ece4d13e1c2de5e9f23dfdb1510fd5e280e95efa2329dda"; };
"@jimp/png@npm:0.16.2" = { filename = "@jimp-png-npm-0.16.2-1ca1111424-c25e0fdd3b.zip"; sha512 = "c25e0fdd3bf296e0df6e514ced95b59135590742a5243ef1070644140aa550311fcc0c4fa9f96d43a850b6499dea7507209c5a3825419bb8c4b25ca4eca8cd92"; };
"koa@npm:2.14.1" = { filename = "koa-npm-2.14.1-92dad86c35-84cfce48d7.zip"; sha512 = "84cfce48d72cf434831bd9f4f06beb3815da030325a7e89b13ee684f5afd158221e16c2d450ca637b221aa0c4a0ab235edd129e4efea656dc5d8384801d3e90f"; };
"err-code@npm:2.0.3" = { filename = "err-code-npm-2.0.3-082e0ff9a7-8b7b1be20d.zip"; sha512 = "8b7b1be20d2de12d2255c0bc2ca638b7af5171142693299416e6a9339bd7d88fc8d7707d913d78e0993176005405a236b066b45666b27b797252c771156ace54"; };
"retry@npm:0.12.0" = { filename = "retry-npm-0.12.0-72ac7fb4cc-623bd7d2e5.zip"; sha512 = "623bd7d2e5119467ba66202d733ec3c2e2e26568074923bc0585b6b99db14f357e79bdedb63cab56cec47491c4a0da7e6021a7465ca6dc4f481d3898fdd3158c"; };
"npmlog@npm:5.0.1" = { filename = "npmlog-npm-5.0.1-366cab64a2-516b266302.zip"; sha512 = "516b2663028761f062d13e8beb3f00069c5664925871a9b57989642ebe09f23ab02145bf3ab88da7866c4e112cafff72401f61a672c7c8a20edc585a7016ef5f"; };
"@jimp/gif@npm:0.16.2" = { filename = "@jimp-gif-npm-0.16.2-a645e3f4b2-0699df7170.zip"; sha512 = "0699df7170b3e11e7a1738205404cfe4a9bf54ad30e405826aa7539c4f389bbca2d9576bc03769e8d52ce474fd19ec392725420fc373fb47986a90fc5680b52e"; };
"color-name@npm:1.1.3" = { filename = "color-name-npm-1.1.3-728b7b5d39-09c5d3e33d.zip"; sha512 = "09c5d3e33d2105850153b14466501f2bfb30324a2f76568a408763a3b7433b0e50e5b4ab1947868e65cb101bb7cb75029553f2c333b6d4b8138a73fcc133d69d"; };
"type@npm:2.6.0" = { filename = "type-npm-2.6.0-bc8cf9ff15-80da01fcc0.zip"; sha512 = "80da01fcc0f6ed5a253dc326530e134000a8f66ea44b6d9687cde2f894f0d0b2486595b0cd040a64f7f79dc3120784236f8c9ef667a8aef03984e049b447cfb4"; };
"semver@npm:6.3.0" = { filename = "semver-npm-6.3.0-b3eace8bfd-1b26ecf6db.zip"; sha512 = "1b26ecf6db9e8292dd90df4e781d91875c0dcc1b1909e70f5d12959a23c7eebb8f01ea581c00783bbee72ceeaad9505797c381756326073850dc36ed284b21b9"; };
"@tootallnate/once@npm:1.1.2" = { filename = "@tootallnate-once-npm-1.1.2-0517220057-e1fb1bbbc1.zip"; sha512 = "e1fb1bbbc12089a0cb9433dc290f97bddd062deadb6178ce9bcb93bb7c1aecde5e60184bc7065aec42fe1663622a213493c48bbd4972d931aae48315f18e1be9"; };
"agent-base@npm:6.0.2" = { filename = "agent-base-npm-6.0.2-428f325a93-f52b6872cc.zip"; sha512 = "f52b6872cc96fd5f622071b71ef200e01c7c4c454ee68bc9accca90c98cfb39f2810e3e9aa330435835eedc8c23f4f8a15267f67c6e245d2b33757575bdac49d"; };
"object-inspect@npm:1.12.3" = { filename = "object-inspect-npm-1.12.3-1e7d20f5ff-dabfd824d9.zip"; sha512 = "dabfd824d97a5f407e6d5d24810d888859f6be394d8b733a77442b277e0808860555176719c5905e765e3743a7cada6b8b0a3b85e5331c530fd418cc8ae991db"; };
"type@npm:1.2.0" = { filename = "type-npm-1.2.0-e67311c4b2-dae8c64f82.zip"; sha512 = "dae8c64f82c648b985caf321e9dd6e8b7f4f2e2d4f846fc6fd2c8e9dc7769382d8a52369ddbaccd59aeeceb0df7f52fb339c465be5f2e543e81e810e413451ee"; };
"humanize-ms@npm:1.2.1" = { filename = "humanize-ms-npm-1.2.1-e942bd7329-9c7a74a282.zip"; sha512 = "9c7a74a2827f9294c009266c82031030eae811ca87b0da3dceb8d6071b9bde22c9f3daef0469c3c533cc67a97d8a167cd9fc0389350e5f415f61a79b171ded16"; };
"tar@npm:6.1.13" = { filename = "tar-npm-6.1.13-3234e72781-8a278bed12.zip"; sha512 = "8a278bed123aa9f53549b256a36b719e317c8b96fe86a63406f3c62887f78267cea9b22dc6f7007009738509800d4a4dccc444abd71d762287c90f35b002eb1c"; };
"@jimp/core@npm:0.16.2" = { filename = "@jimp-core-npm-0.16.2-240819bdf1-16bcc2be91.zip"; sha512 = "16bcc2be91c86ee3b523af98b4f08cb159f2705761b67cc5209c2d97d98f9a6f8b25f80995ca58f8665f43c068e05d8cf19018206fc8ec41100b1c177c506406"; };
"bmp-js@npm:0.1.0" = { filename = "bmp-js-npm-0.1.0-5c9f284ca3-2f6cf7eeab.zip"; sha512 = "2f6cf7eeabae2aa50eb768122f59e9752caa97248028cb8b5cf0d9db7cf8fb3a60262aeb2c6889dd21357ab061b2fb318f21f20d2b24963ba36ead8e264c6654"; };
"jpeg-js@npm:0.4.4" = { filename = "jpeg-js-npm-0.4.4-4dd87659c3-bd7cb61aa8.zip"; sha512 = "bd7cb61aa8df40a9ee2c2106839c3df6054891e56cfc22c0ac581402e06c6295f962a4754b0b2ac50a401789131b1c6dc9df8d24400f1352168be1894833c590"; };
"pngjs@npm:3.4.0" = { filename = "pngjs-npm-3.4.0-4e495c1dad-8bd40bd698.zip"; sha512 = "8bd40bd698abd16b72c97b85cb858c80894fbedc76277ce72a784aa441e14795d45d9856e97333ca469b34b67528860ffc8a7317ca6beea349b645366df00bcd"; };
"utif@npm:2.0.1" = { filename = "utif-npm-2.0.1-b6892ce22a-66b0bffc18.zip"; sha512 = "66b0bffc18f08834a34c44846f189b2223418e8bddfff3882e07fccea54436737f7334e27c86cbff46e247487dc92377b7a17428a7aafc5d5c733d61bed39038"; };
"http-errors@npm:2.0.0" = { filename = "http-errors-npm-2.0.0-3f1c503428-9b0a378266.zip"; sha512 = "9b0a3782665c52ce9dc658a0d1560bcb0214ba5699e4ea15aefb2a496e2ca83db03ebc42e1cce4ac1f413e4e0d2d736a3fd755772c556a9a06853ba2a0b7d920"; };
"encoding@npm:0.1.13" = { filename = "encoding-npm-0.1.13-82a1837d30-bb98632f8f.zip"; sha512 = "bb98632f8ffa823996e508ce6a58ffcf5856330fde839ae42c9e1f436cc3b5cc651d4aeae72222916545428e54fd0f6aa8862fd8d25bdbcc4589f1e3f3715e7f"; };
"minipass-sized@npm:1.0.3" = { filename = "minipass-sized-npm-1.0.3-306d86f432-79076749fc.zip"; sha512 = "79076749fcacf21b5d16dd596d32c3b6bf4d6e62abb43868fac21674078505c8b15eaca4e47ed844985a4514854f917d78f588fcd029693709417d8f98b2bd60"; };
"socks@npm:2.6.2" = { filename = "socks-npm-2.6.2-94c1dcb8b8-dd91942930.zip"; sha512 = "dd9194293059d737759d5c69273850ad4149f448426249325c4bea0e340d1cf3d266c3b022694b0dcf5d31f759de23657244c481fc1e8322add80b7985c36b5e"; };
"call-bind@npm:1.0.2" = { filename = "call-bind-npm-1.0.2-c957124861-f8e31de9d1.zip"; sha512 = "f8e31de9d19988a4b80f3e704788c4a2d6b6f3d17cfec4f57dc29ced450c53a49270dc66bf0fbd693329ee948dd33e6c90a329519aef17474a4d961e8d6426b0"; };
"content-type@npm:1.0.4" = { filename = "content-type-npm-1.0.4-3b1a5ca16b-3d93585fda.zip"; sha512 = "3d93585fda985d1554eca5ebd251994327608d2e200978fdbfba21c0c679914d5faf266d17027de44b34a72c7b0745b18584ecccaa7e1fdfb6a68ac7114f12e0"; };
"depd@npm:2.0.0" = { filename = "depd-npm-2.0.0-b6c51a4b43-abbe19c768.zip"; sha512 = "abbe19c768c97ee2eed6282d8ce3031126662252c58d711f646921c9623f9052e3e1906443066beec1095832f534e57c523b7333f8e7e0d93051ab6baef5ab3a"; };
"destroy@npm:1.2.0" = { filename = "destroy-npm-1.2.0-6a511802e2-0acb300b74.zip"; sha512 = "0acb300b7478a08b92d810ab229d5afe0d2f4399272045ab22affa0d99dbaf12637659411530a6fcd597a9bdac718fc94373a61a95b4651bbc7b83684a565e38"; };
"encodeurl@npm:1.0.2" = { filename = "encodeurl-npm-1.0.2-f8c8454c41-e50e3d508c.zip"; sha512 = "e50e3d508cdd9c4565ba72d2012e65038e5d71bdc9198cb125beb6237b5b1ade6c0d343998da9e170fb2eae52c1bed37d4d6d98a46ea423a0cddbed5ac3f780c"; };
"escape-html@npm:1.0.3" = { filename = "escape-html-npm-1.0.3-376c22ee74-6213ca9ae0.zip"; sha512 = "6213ca9ae00d0ab8bccb6d8d4e0a98e76237b2410302cf7df70aaa6591d509a2a37ce8998008cbecae8fc8ffaadf3fb0229535e6a145f3ce0b211d060decbb24"; };
"fresh@npm:0.5.2" = { filename = "fresh-npm-0.5.2-ad2bb4c0a2-13ea8b08f9.zip"; sha512 = "13ea8b08f91e669a64e3ba3a20eb79d7ca5379a81f1ff7f4310d54e2320645503cc0c78daedc93dfb6191287295f6479544a649c64d8e41a1c0fb0c221552346"; };
"koa-compose@npm:4.1.0" = { filename = "koa-compose-npm-4.1.0-b682d9324e-46cb16792d.zip"; sha512 = "46cb16792d96425e977c2ae4e5cb04930280740e907242ec9c25e3fb8b4a1d7b54451d7432bc24f40ec62255edea71894d2ceeb8238501842b4e48014f2e83db"; };
"only@npm:0.0.2" = { filename = "only-npm-0.0.2-122402a3f9-d399710db8.zip"; sha512 = "d399710db867a1ef436dd3ce74499c87ece794aa81ab0370b5d153968766ee4aed2f98d3f92fc87c963e45b7a74d400d6f463ef651a5e7cfb861b15e88e9efe6"; };
"parseurl@npm:1.3.3" = { filename = "parseurl-npm-1.3.3-1542397e00-407cee8e0a.zip"; sha512 = "407cee8e0a3a4c5cd472559bca8b6a45b82c124e9a4703302326e9ab60fc1081442ada4e02628efef1eb16197ddc7f8822f5a91fd7d7c86b51f530aedb17dfa2"; };
"vary@npm:1.1.2" = { filename = "vary-npm-1.1.2-b49f70ae63-ae0123222c.zip"; sha512 = "ae0123222c6df65b437669d63dfa8c36cee20a504101b2fcd97b8bf76f91259c17f9f2b4d70a1e3c6bbcee7f51b28392833adb6b2770b23b01abec84e369660b"; };
"omggif@npm:1.0.10" = { filename = "omggif-npm-1.0.10-6ffa5bf343-15102e46b6.zip"; sha512 = "15102e46b6fa0fba32d7e948f702623cdc3cdcdfd64b2d33c6e29a61f366ffd0f250da55d66f5217dce5b93ba9c67763fa998652791a5c7f2201a3bde2c4db45"; };
"jsdom@npm:16.7.0" = { filename = "jsdom-npm-16.7.0-216c5c4bf9-454b833718.zip"; sha512 = "454b83371857000763ed31130a049acd1b113e3b927e6dcd75c67ddc30cdd242d7ebcac5c2294b7a1a6428155cb1398709c573b3c6d809218692ea68edd93370"; };
"get-intrinsic@npm:1.1.3" = { filename = "get-intrinsic-npm-1.1.3-b6c7bd1010-152d79e872.zip"; sha512 = "152d79e87251d536cf880ba75cfc3d6c6c50e12b3a64e1ea960e73a3752b47c69f46034456eae1b0894359ce3bc64c55c186f2811f8a788b75b638b06fab228a"; };
"content-disposition@npm:0.5.4" = { filename = "content-disposition-npm-0.5.4-2d93678616-afb9d545e2.zip"; sha512 = "afb9d545e296a5171d7574fcad634b2fdf698875f4006a9dd04a3e1333880c5c0c98d47b560d01216fb6505a54a2ba6a843ee3a02ec86d7e911e8315255f56c3"; };
"is-generator-function@npm:1.0.10" = { filename = "is-generator-function-npm-1.0.10-1d0f3809ef-d54644e7db.zip"; sha512 = "d54644e7dbaccef15ceb1e5d91d680eb5068c9ee9f9eb0a9e04173eb5542c9b51b5ab52c5537f5703e48d5fddfd376817c1ca07a84a407b7115b769d4bdde72b"; };
"on-finished@npm:2.4.1" = { filename = "on-finished-npm-2.4.1-907af70f88-d20929a25e.zip"; sha512 = "d20929a25e7f0bb62f937a425b5edeb4e4cde0540d77ba146ec9357f00b0d497cdb3b9b05b9c8e46222407d1548d08166bff69cc56dfa55ba0e4469228920ff0"; };
"@jimp/utils@npm:0.16.2" = { filename = "@jimp-utils-npm-0.16.2-a90a847301-96f3c815f3.zip"; sha512 = "96f3c815f3ac5bd16ab7cdfe97eef9839e070fca6d15ca171dbf934a8ab9af17ad4c861445ca69d0f00a4ea5fe70303efbde77325c5b181db5441d6bfa96b444"; };
"accepts@npm:1.3.8" = { filename = "accepts-npm-1.3.8-9a812371c9-50c43d32e7.zip"; sha512 = "50c43d32e7b50285ebe84b613ee4a3aa426715a7d131b65b786e2ead0fd76b6b60091b9916d3478a75f11f162628a2139991b6c03ab3f1d9ab7c86075dc8eab4"; };
"cache-content-type@npm:1.0.1" = { filename = "cache-content-type-npm-1.0.1-f709f8c309-18db4d5945.zip"; sha512 = "18db4d59452669ccbfd7146a1510a37eb28e9eccf18ca7a4eb603dff2edc5cccdca7498fc3042a2978f76f11151fba486eb9eb69d9afa3fb124957870aef4fd3"; };
"cookies@npm:0.8.0" = { filename = "cookies-npm-0.8.0-d7388cbd94-806055a44f.zip"; sha512 = "806055a44f128705265b1bc6a853058da18bf80dea3654ad99be20985b1fa1b14f86c1eef73644aab8071241f8a78acd57202b54c4c5c70769fc694fbb9c4edc"; };
"http-assert@npm:1.5.0" = { filename = "http-assert-npm-1.5.0-bf7ea4ffcf-69c9b3c14c.zip"; sha512 = "69c9b3c14cf8b2822916360a365089ce936c883c49068f91c365eccba5c141a9964d19fdda589150a480013bf503bf37d8936c732e9635819339e730ab0e7527"; };
"koa-convert@npm:2.0.0" = { filename = "koa-convert-npm-2.0.0-d709eca55c-7385b33919.zip"; sha512 = "7385b3391995f59c1312142e110d5dff677f9850dbfbcf387cd36a7b0af03b5d26e82b811eb9bb008b4f3e661cdab1f8817596e46b1929da2cf6e97a2f7456ed"; };
"are-we-there-yet@npm:2.0.0" = { filename = "are-we-there-yet-npm-2.0.0-7d2f5201ce-6c80b4fd04.zip"; sha512 = "6c80b4fd04ecee6ba6e737e0b72a4b41bdc64b7d279edfc998678567ff583c8df27e27523bc789f2c99be603ffa9eaa612803da1d886962d2086e7ff6fa90c7c"; };
"any-base@npm:1.1.0" = { filename = "any-base-npm-1.1.0-1f3e797fba-c1fd040de5.zip"; sha512 = "c1fd040de52e710e2de7d9ae4df52bac589f35622adb24686c98ce21c7b824859a8db9614bc119ed8614f42fd08918b2612e6a6c385480462b3100a1af59289d"; };
"exif-parser@npm:0.1.12" = { filename = "exif-parser-npm-0.1.12-ff077373a4-6ba50cb9e0.zip"; sha512 = "6ba50cb9e0b45a6efa37e966a9582ecd171b5c5b3ef0c47542f2b862c521f70d2f656dde85b4d2a5dd8e1163486b09049f4c412e9c6176bfbda1654a5b2f021c"; };
"file-type@npm:9.0.0" = { filename = "file-type-npm-9.0.0-e0f2c352e4-9ea78b29c3.zip"; sha512 = "9ea78b29c3762d967eb1e3e4f45e401388b6d252b12c217f78f5ea97556ff7e35e4c7255cab68810ac414d51b776bd4e83504c86f132c262a454251561189efa"; };
"phin@npm:2.9.3" = { filename = "phin-npm-2.9.3-8661be52b1-7e2abd7be7.zip"; sha512 = "7e2abd7be74a54eb7be92dccb1d7a019725c8adaa79ac22a38f25220f9a859393e654ea753a559d326aed7bbc966fadac88270cc8c39d78896f7784219560c47"; };
"tinycolor2@npm:1.5.2" = { filename = "tinycolor2-npm-1.5.2-0898381e92-9df1ea9a98.zip"; sha512 = "9df1ea9a986b03f1aebb1c1ac17fc561e358493f61b56d73ef2d7207fe7bd74eb71cf745b70487b2b5bb1ce33c9e8af7101088bb0b5fc532eaa1f9d1eda4ef31"; };
"gifwrap@npm:0.9.4" = { filename = "gifwrap-npm-0.9.4-f819bc557d-f06d74d72d.zip"; sha512 = "f06d74d72d5fd5cfef78935471da06334c23305328fcaa5b1740c6c93b7ce7f775c28e75e166ecb4d7bc9848f767d4d191c8e18a74187543076a44123002f918"; };
"pako@npm:1.0.11" = { filename = "pako-npm-1.0.11-b8f1b69d3e-1be2bfa1f8.zip"; sha512 = "1be2bfa1f807608c7538afa15d6f25baa523c30ec870a3228a89579e474a4d992f4293859524e46d5d87fd30fa17c5edf34dbef0671251d9749820b488660b16"; };
"statuses@npm:2.0.1" = { filename = "statuses-npm-2.0.1-81d2b97fee-18c7623fdb.zip"; sha512 = "18c7623fdb8f646fb213ca4051be4df7efb3484d4ab662937ca6fbef7ced9b9e12842709872eb3020cc3504b93bde88935c9f6417489627a7786f24f8031cbcb"; };
"ip@npm:1.1.5" = { filename = "ip-npm-1.1.5-af36318aa6-30133981f0.zip"; sha512 = "30133981f082a060a32644f6a7746e9ba7ac9e2bc07ecc8bbdda3ee8ca9bec1190724c390e45a1ee7695e7edfd2a8f7dda2c104ec5f7ac5068c00648504c7e5a"; };
"minipass@npm:4.0.0" = { filename = "minipass-npm-4.0.0-2520d5ae3c-7a609afbf3.zip"; sha512 = "7a609afbf394abfcf9c48e6c90226f471676c8f2a67f07f6838871afb03215ede431d1433feffe1b855455bcb13ef0eb89162841b9796109d6fed8d89790f381"; };
"mkdirp@npm:0.5.6" = { filename = "mkdirp-npm-0.5.6-dcd5a6b97b-0c91b721bb.zip"; sha512 = "0c91b721bb12c3f9af4b77ebf73604baf350e64d80df91754dc509491ae93bf238581e59c7188360cec7cb62fc4100959245a42cfe01834efedc5e9d068376c2"; };
"pixelmatch@npm:4.0.2" = { filename = "pixelmatch-npm-4.0.2-03016768a7-9c5c132900.zip"; sha512 = "9c5c1329001938cae6d01e2bb84a909ba767f8256bcafc075422cea2a4dbaa8bebd44fceaa4b4ce7cdc36d11f20d4f1ba0cf669851d5649b32d8d1d27e4f5a36"; };
"function-bind@npm:1.1.1" = { filename = "function-bind-npm-1.1.1-b56b322ae9-b32fbaebb3.zip"; sha512 = "b32fbaebb3f8ec4969f033073b43f5c8befbb58f1a79e12f1d7490358150359ebd92f49e72ff0144f65f2c48ea2a605bff2d07965f548f6474fd8efd95bf361a"; };
"abab@npm:2.0.6" = { filename = "abab-npm-2.0.6-2662fba7f0-6ffc1af4ff.zip"; sha512 = "6ffc1af4ff315066c62600123990d87551ceb0aafa01e6539da77b0f5987ac7019466780bf480f1787576d4385e3690c81ccc37cfda12819bf510b8ab47e5a3e"; };
"acorn@npm:8.8.1" = { filename = "acorn-npm-8.8.1-20e4aea981-4079b67283.zip"; sha512 = "4079b67283b94935157698831967642f24a075c52ce3feaaaafe095776dfbe15d86a1b33b1e53860fc0d062ed6c83f4284a5c87c85b9ad51853a01173da6097f"; };
"cssom@npm:0.4.4" = { filename = "cssom-npm-0.4.4-818f01a6e3-e3bc1076e7.zip"; sha512 = "e3bc1076e7ee4213d4fef05e7ae03bfa83dc05f32611d8edc341f4ecc3d9647b89c8245474c7dd2cdcdb797a27c462e99da7ad00a34399694559f763478ff53f"; };
"decimal.js@npm:10.4.3" = { filename = "decimal.js-npm-10.4.3-e7d483387c-796404dcfa.zip"; sha512 = "796404dcfa9d1dbfdc48870229d57f788b48c21c603c3f6554a1c17c10195fc1024de338b0cf9e1efe0c7c167eeb18f04548979bcc5fdfabebb7cc0ae3287bae"; };
"is-potential-custom-element-name@npm:1.0.1" = { filename = "is-potential-custom-element-name-npm-1.0.1-f352f606f8-ced7bbbb64.zip"; sha512 = "ced7bbbb6433a5b684af581872afe0e1767e2d1146b2207ca0068a648fb5cab9d898495d1ac0583524faaf24ca98176a7d9876363097c2d14fee6dd324f3a1ab"; };
"nwsapi@npm:2.2.2" = { filename = "nwsapi-npm-2.2.2-b77040326c-4376910629.zip"; sha512 = "43769106292bc95f776756ca2f3513dab7b4d506a97c67baec32406447841a35f65f29c1f95ab5d42785210fd41668beed33ca16fa058780be43b101ad73e205"; };
"parse5@npm:6.0.1" = { filename = "parse5-npm-6.0.1-70a35a494a-7d569a176c.zip"; sha512 = "7d569a176c5460897f7c8f3377eff640d54132b9be51ae8a8fa4979af940830b2b0c296ce75e5bd8f4041520aadde13170dbdec44889975f906098ea0002f4bd"; };
"symbol-tree@npm:3.2.4" = { filename = "symbol-tree-npm-3.2.4-fe70cdb75b-6e8fc7e148.zip"; sha512 = "6e8fc7e1486b8b54bea91199d9535bb72f10842e40c79e882fc94fb7b14b89866adf2fd79efa5ebb5b658bc07fb459ccce5ac0e99ef3d72f474e74aaf284029d"; };
"webidl-conversions@npm:6.1.0" = { filename = "webidl-conversions-npm-6.1.0-0594fd577c-1f526507aa.zip"; sha512 = "1f526507aa491f972a0c1409d07f8444e1d28778dfa269a9971f2e157182f3d496dc33296e4ed45b157fdb3bf535bb90c90bf10c50dcf1dd6caacb2a34cc84fb"; };
"whatwg-mimetype@npm:2.3.0" = { filename = "whatwg-mimetype-npm-2.3.0-52eaa1d941-23eb885940.zip"; sha512 = "23eb885940bcbcca4ff841c40a78e9cbb893ec42743993a42bf7aed16085b048b44b06f3402018931687153550f9a32d259dfa524e4f03577ab898b6965e5383"; };
"ws@npm:7.5.9" = { filename = "ws-npm-7.5.9-26f12a5ed6-c3c100a181.zip"; sha512 = "c3c100a181b731f40b7f2fddf004aa023f79d64f489706a28bc23ff88e87f6a64b3c6651fbec3a84a53960b75159574d7a7385709847a62ddb7ad6af76f49138"; };
"xml-name-validator@npm:3.0.0" = { filename = "xml-name-validator-npm-3.0.0-10e74a38ea-b3ac459afe.zip"; sha512 = "b3ac459afed783c285bb98e4960bd1f3ba12754fd4f2320efa0f9181ca28928c53cc75ca660d15d205e81f92304419afe94c531c7cfb3e0649aa6d140d53ecb0"; };
"iconv-lite@npm:0.6.3" = { filename = "iconv-lite-npm-0.6.3-24b8aae27e-3f60d47a5c.zip"; sha512 = "3f60d47a5c8fc3313317edfd29a00a692cc87a19cac0159e2ce711d0ebc9019064108323b5e493625e25594f11c6236647d8e256fbe7a58f4a3b33b89e6d30bf"; };
"has-symbols@npm:1.0.3" = { filename = "has-symbols-npm-1.0.3-1986bff2c4-a054c40c63.zip"; sha512 = "a054c40c631c0d5741a8285010a0777ea0c068f99ed43e5d6eb12972da223f8af553a455132fdb0801bdcfa0e0f443c0c03a68d8555aa529b3144b446c3f2410"; };
"ee-first@npm:1.1.1" = { filename = "ee-first-npm-1.1.1-33f8535b39-1b4cac778d.zip"; sha512 = "1b4cac778d64ce3b582a7e26b218afe07e207a0f9bfe13cc7395a6d307849cfe361e65033c3251e00c27dd060cab43014c2d6b2647676135e18b77d2d05b3f4f"; };
"@jimp/plugins@npm:0.16.2" = { filename = "@jimp-plugins-npm-0.16.2-58fd5e1add-b259190581.zip"; sha512 = "b25919058104451ae859ede6c385324da06c29c54a8e7348ada68d149dd53a197614fbf188aa7a46b44fa9301a402b69d2345ca25ba7c7e29602cf4a178d0496"; };
"cssstyle@npm:2.3.0" = { filename = "cssstyle-npm-2.3.0-b5d112c450-5f05e6fd2e.zip"; sha512 = "5f05e6fd2e3df0b44695c2f08b9ef38b011862b274e320665176467c0725e44a53e341bc4959a41176e83b66064ab786262e7380fd1cabeae6efee0d255bb4e3"; };
"domexception@npm:2.0.1" = { filename = "domexception-npm-2.0.1-81b20626ae-d638e9cb05.zip"; sha512 = "d638e9cb05c52999f1b2eb87c374b03311ea5b1d69c2f875bc92da73e17db60c12142b45c950228642ff7f845c536b65305483350d080df59003a653da80b691"; };
"html-encoding-sniffer@npm:2.0.1" = { filename = "html-encoding-sniffer-npm-2.0.1-381bf15a76-bf30cce461.zip"; sha512 = "bf30cce461015ed7e365736fcd6a3063c7bc016a91f74398ef6158886970a96333938f7c02417ab3c12aa82e3e53b40822145facccb9ddfbcdc15a879ae4d7ba"; };
"saxes@npm:5.0.1" = { filename = "saxes-npm-5.0.1-57abf031ae-5636b55cf1.zip"; sha512 = "5636b55cf15f7cf0baa73f2797bf992bdcf75d1b39d82c0aa4608555c774368f6ac321cb641fd5f3d3ceb87805122cd47540da6a7b5960fe0dbdb8f8c263f000"; };
"w3c-hr-time@npm:1.0.2" = { filename = "w3c-hr-time-npm-1.0.2-87f88e51d9-ec3c2dacbf.zip"; sha512 = "ec3c2dacbf8050d917bbf89537a101a08c2e333b4c19155f7d3bedde43529d4339db6b3d049d9610789cb915f9515f8be037e0c54c079e9d4735c50b37ed52b9"; };
"w3c-xmlserializer@npm:2.0.0" = { filename = "w3c-xmlserializer-npm-2.0.0-f8f7bc8b42-ae25c51cf7.zip"; sha512 = "ae25c51cf71f1fb2516df1ab33a481f83461a117565b95e3d0927432522323f93b1b2846cbb60196d337970c421adb604fc2d0d180c6a47a839da01db5b9973b"; };
"whatwg-encoding@npm:1.0.5" = { filename = "whatwg-encoding-npm-1.0.5-85e0fb7d7d-5be4efe111.zip"; sha512 = "5be4efe111dce29ddee3448d3915477fcc3b28f991d9cf1300b4e50d6d189010d47bca2f51140a844cf9b726e8f066f4aee72a04d687bfe4f2ee2767b2f5b1e6"; };
"has@npm:1.0.3" = { filename = "has-npm-1.0.3-b7f00631c1-b9ad53d53b.zip"; sha512 = "b9ad53d53be4af90ce5d1c38331e712522417d017d5ef1ebd0507e07c2fbad8686fffb8e12ddecd4c39ca9b9b47431afbb975b8abf7f3c3b82c98e9aad052792"; };
"ylru@npm:1.3.2" = { filename = "ylru-npm-1.3.2-81969d097f-b6bb393114.zip"; sha512 = "b6bb3931144424114f2350c072cfeb180f205add93509c605ae025cbed8059846f8a5767655feeeab890d288b5b4c4b36f5d5d867ee4e6946c16bcc7ec3ddaee"; };
"deep-equal@npm:1.0.1" = { filename = "deep-equal-npm-1.0.1-f05565c4e5-5af8cbfceb.zip"; sha512 = "5af8cbfcebf190491878a498caccc7dc9592f8ebd1685b976eacc3825619d222b5e929923163b92c4f414494e2b884f7ebf00c022e8198e8292deb70dd9785f4"; };
"has-tostringtag@npm:1.0.0" = { filename = "has-tostringtag-npm-1.0.0-b1fcf3ab55-cc12eb28cb.zip"; sha512 = "cc12eb28cb6ae22369ebaad3a8ab0799ed61270991be88f208d508076a1e99abe4198c965935ce85ea90b60c94ddda73693b0920b58e7ead048b4a391b502c1c"; };
"co@npm:4.6.0" = { filename = "co-npm-4.6.0-03f2d1feb6-5210d92230.zip"; sha512 = "5210d9223010eb95b29df06a91116f2cf7c8e0748a9013ed853b53f362ea0e822f1e5bb054fb3cefc645239a4cf966af1f6133a3b43f40d591f3b68ed6cf0510"; };
"acorn-globals@npm:6.0.0" = { filename = "acorn-globals-npm-6.0.0-acbec28ad5-72d95e5b5e.zip"; sha512 = "72d95e5b5e585f9acd019b993ab8bbba68bb3cbc9d9b5c1ebb3c2f1fe5981f11deababfb4949f48e6262f9c57878837f5958c0cca396f81023814680ca878042"; };
"keygrip@npm:1.1.0" = { filename = "keygrip-npm-1.1.0-8da030c3ff-078cd16a46.zip"; sha512 = "078cd16a463d187121f0a27c1c9c95c52ad392b620f823431689f345a0501132cee60f6e96914b07d570105af470b96960402accd6c48a0b1f3cd8fac4fa2cae"; };
"minimist@npm:1.2.7" = { filename = "minimist-npm-1.2.7-51d33b1371-7346574a10.zip"; sha512 = "7346574a1038ca23c32e02252f603801f09384dd1d78b69a943a4e8c2c28730b80e96193882d3d3b22a063445f460e48316b29b8a25addca2d7e5e8f75478bec"; };
"image-q@npm:4.0.0" = { filename = "image-q-npm-4.0.0-22657645c0-6c6a1dd846.zip"; sha512 = "6c6a1dd8467833161f46cc17b4a43218d30a3899ff71a4ffb895f71a14ca29de12f79d824d4d2174070924cbd97faa018b2ded8d690483ab7eb269f364cd97cc"; };
"data-urls@npm:2.0.0" = { filename = "data-urls-npm-2.0.0-2b80c32b82-97caf828aa.zip"; sha512 = "97caf828aac25e25e04ba6869db0f99c75e6859bb5b424ada28d3e7841941ebf08ddff3c1b1bb4585986bd507a5d54c2a716853ea6cb98af877400e637393e71"; };
"form-data@npm:3.0.1" = { filename = "form-data-npm-3.0.1-d080d436e0-b019e8d35c.zip"; sha512 = "b019e8d35c8afc14a2bd8a7a92fa4f525a4726b6d5a9740e8d2623c30e308fbb58dc8469f90415a856698933c8479b01646a9dff33c87cc4e76d72aedbbf860d"; };
"whatwg-url@npm:8.7.0" = { filename = "whatwg-url-npm-8.7.0-67af66db8f-a87abcc6ce.zip"; sha512 = "a87abcc6cefcece5311eb642858c8fdb234e51ec74196bfacf8def2edae1bfbffdf6acb251646ed6301f8cee44262642d8769c707256125a91387e33f405dd1e"; };
"tough-cookie@npm:4.1.2" = { filename = "tough-cookie-npm-4.1.2-09068826e1-a7359e9a3e.zip"; sha512 = "a7359e9a3e875121a84d6ba40cc184dec5784af84f67f3a56d1d2ae39b87c0e004e6ba7c7331f9622a7d2c88609032473488b28fe9f59a1fec115674589de39a"; };
"cssom@npm:0.3.8" = { filename = "cssom-npm-0.3.8-a9291d36ff-24beb3087c.zip"; sha512 = "24beb3087c76c0d52dd458be9ee1fbc80ac771478a9baef35dd258cdeb527c68eb43204dd439692bb2b1ae5272fa5f2946d10946edab0d04f1078f85e06bc7f6"; };
"webidl-conversions@npm:5.0.0" = { filename = "webidl-conversions-npm-5.0.0-9649787484-ccf1ec2ca7.zip"; sha512 = "ccf1ec2ca7c0b5671e5440ace4a66806ae09c49016ab821481bec0c05b1b82695082dc0a27d1fe9d804d475a408ba0c691e6803fd21be608e710955d4589cd69"; };
"xmlchars@npm:2.2.0" = { filename = "xmlchars-npm-2.2.0-8b78f0f5e4-8c70ac9407.zip"; sha512 = "8c70ac94070ccca03f47a81fcce3b271bd1f37a591bf5424e787ae313fcb9c212f5f6786e1fa82076a2c632c0141552babcd85698c437506dfa6ae2d58723062"; };
"browser-process-hrtime@npm:1.0.0" = { filename = "browser-process-hrtime-npm-1.0.0-db700805c2-e30f868cdb.zip"; sha512 = "e30f868cdb770b1201afb714ad1575dd86366b6e861900884665fb627109b3cc757c40067d3bfee1ff2a29c835257ea30725a8018a9afd02ac1c24b408b1e45f"; };
"cacache@npm:15.3.0" = { filename = "cacache-npm-15.3.0-a7e5239c6a-a07327c27a.zip"; sha512 = "a07327c27a4152c04eb0a831c63c00390d90f94d51bb80624a66f4e14a6b6360bbf02a84421267bd4d00ca73ac9773287d8d7169e8d2eafe378d2ce140579db8"; };
"escodegen@npm:2.0.0" = { filename = "escodegen-npm-2.0.0-6450b02925-5aa6b2966f.zip"; sha512 = "5aa6b2966fafe0545e4e77936300cc94ad57cfe4dc4ebff9950492eaba83eef634503f12d7e3cbd644ecc1bab388ad0e92b06fd32222c9281a75d1cf02ec6cef"; };
"gauge@npm:3.0.2" = { filename = "gauge-npm-3.0.2-9e22f7af9e-81296c00c7.zip"; sha512 = "81296c00c7410cdd48f997800155fbead4f32e4f82109be0719c63edc8560e6579946cc8abd04205297640691ec26d21b578837fd13a4e96288ab4b40b1dc3e9"; };
"acorn-walk@npm:7.2.0" = { filename = "acorn-walk-npm-7.2.0-5f8b515308-9252158a79.zip"; sha512 = "9252158a79b9d92f1bc0dd6acc0fcfb87a67339e84bcc301bb33d6078936d27e35d606b4d35626d2962cd43c256d6f27717e70cbe15c04fff999ab0b2260b21f"; };
"acorn@npm:7.4.1" = { filename = "acorn-npm-7.4.1-f450b4646c-1860f23c21.zip"; sha512 = "1860f23c2107c910c6177b7b7be71be350db9e1080d814493fae143ae37605189504152d1ba8743ba3178d0b37269ce1ffc42b101547fdc1827078f82671e407"; };
"load-bmfont@npm:1.4.1" = { filename = "load-bmfont-npm-1.4.1-f3f8c0beeb-688d932fb0.zip"; sha512 = "688d932fb0dc4c9333747736ccd926261f0b91734b7bdb6ff24f8659ef068a0f0b2278084b208851afac0beec79af7bd6664fe2ed5b6c5e1db88755fc25f785e"; };
"tsscmp@npm:1.0.6" = { filename = "tsscmp-npm-1.0.6-3223087558-1512384def.zip"; sha512 = "1512384def36bccc9125cabbd4c3b0e68608d7ee08127ceaa0b84a71797263f1a01c7f82fa69be8a3bd3c1396e2965d2f7b52d581d3a5eeaf3967fbc52e3b3bf"; };
"@jimp/plugin-blit@npm:0.16.2" = { filename = "@jimp-plugin-blit-npm-0.16.2-ecd1517d17-9cfac881be.zip"; sha512 = "9cfac881bed691822d763184922ca3055134946d771af7a0b610b67d36d3d5981fe97bc440e7f499eda14189a06940978b3e956a780438c1598c353f39a628da"; };
"@jimp/plugin-blur@npm:0.16.2" = { filename = "@jimp-plugin-blur-npm-0.16.2-ca832b7222-9b8a9d70dd.zip"; sha512 = "9b8a9d70dd7906500b4162436d99f46d97290e950475a94b2c55250e196484ec09b04147164ac37d80decb90d5d0c631dd3e980730bae67b6b4bb5f9d445975a"; };
"@jimp/plugin-circle@npm:0.16.2" = { filename = "@jimp-plugin-circle-npm-0.16.2-e0dc828fb5-ea15dfede0.zip"; sha512 = "ea15dfede0b0042ef6cdcdfdbaa4eb4138ba580779e3f7f1d3702505631f3be960a42c32f7798fde0a77235529e1b1624678db63669992e3ec9c329d0f426210"; };
"@jimp/plugin-contain@npm:0.16.2" = { filename = "@jimp-plugin-contain-npm-0.16.2-54bacf252b-6684cb995c.zip"; sha512 = "6684cb995cac69c946b746d3fb82069ca96a67914f32682be6ffb9305064f1f5729796914c98ec93466b3e0928de69488bd079d8adc130868ceac2efa0526301"; };
"@jimp/plugin-cover@npm:0.16.2" = { filename = "@jimp-plugin-cover-npm-0.16.2-4f62c6454b-0053f89910.zip"; sha512 = "0053f8991079fa70a5bb9e47ba75a878627f691769625f8122627710de04d77af1dbd3d377ae0d5201daefd41fec602acb4ebe6ac56caa7c196a1bb0fadded34"; };
"@jimp/plugin-crop@npm:0.16.2" = { filename = "@jimp-plugin-crop-npm-0.16.2-75a95e46d8-abf31e3c18.zip"; sha512 = "abf31e3c1899a241606555ae140fb32004e3abf94398377016d9b86117d8865738763a807c55998af07cd63481a66fdc75a7546bf784a1ec080e22dc8675c0ec"; };
"@jimp/plugin-displace@npm:0.16.2" = { filename = "@jimp-plugin-displace-npm-0.16.2-2da15a6c34-92e48e5bf2.zip"; sha512 = "92e48e5bf2d8bb930933d0d750fa0ada5654e0282c94a4e04b583fee24b2677b72338efba388d6716776543e32d84bf26e38381aa5e15451080f060ecfd93c54"; };
"@jimp/plugin-dither@npm:0.16.2" = { filename = "@jimp-plugin-dither-npm-0.16.2-5aa568ca99-2dd3b81cfb.zip"; sha512 = "2dd3b81cfbf9e01620fbde10a9345097d151888513e5f3f2ca7ab51296a9765ca88f1e452a420db7bd0f9917ae9521185792848e15a26f8f37c85c4d09c8e65e"; };
"@jimp/plugin-fisheye@npm:0.16.2" = { filename = "@jimp-plugin-fisheye-npm-0.16.2-eb455d75a5-ebe8a14b92.zip"; sha512 = "ebe8a14b926d4950b9777e0603bdd2ebaa19b0ec7bc3e5bfc6291a972baec5f6f23115fad190ccb1a6b9c80d4ad3914e357639e2900a65c69748cf3d021ddab6"; };
"@jimp/plugin-flip@npm:0.16.2" = { filename = "@jimp-plugin-flip-npm-0.16.2-0645289d39-c8b6646b16.zip"; sha512 = "c8b6646b168b9ea0f549121369062c039c6dd7e277f646577aedf86c501f99c9f0a4b27ba8db7b6ea67c38f4adc43d5f5df90e02fdb90aab39476dca051547be"; };
"@jimp/plugin-gaussian@npm:0.16.2" = { filename = "@jimp-plugin-gaussian-npm-0.16.2-d0fc9a0338-b768e18e32.zip"; sha512 = "b768e18e32b5b4bba9b3bf95cf9f84cce6ef2070d8b17f1761282941e8ad8f216f5fb4390a09fde9bcc1a1be6f6f06256db3e74222b7745635da63251f749026"; };
"@jimp/plugin-invert@npm:0.16.2" = { filename = "@jimp-plugin-invert-npm-0.16.2-1ffeb5719e-f68302f322.zip"; sha512 = "f68302f322b7668ecf3c0e66ff742ce1a4a2fdaa4443a7a95c7909b73f7a12095b6b308f7f6ae812d434bb2d5ede4879815dc7a0caf1d8cae3aff03b01a36fdb"; };
"@jimp/plugin-mask@npm:0.16.2" = { filename = "@jimp-plugin-mask-npm-0.16.2-30dc3d3e7d-cee7c3b7c9.zip"; sha512 = "cee7c3b7c96ca360d7b4e94605a53dbf06ca2613fe165cf880b931a744fdaaa492fdae16314b37ce5938d78371c25e7d70db9b721a182e3803219f0d51c41ae2"; };
"@jimp/plugin-normalize@npm:0.16.2" = { filename = "@jimp-plugin-normalize-npm-0.16.2-5a1901f58c-79680c752b.zip"; sha512 = "79680c752bbd4b271dcb56bf23279cd0eb4f7d31ff55d89d3439a785e6c556c7afdf1be0454b54ce8f47d78f09c32f8da4eb5e40ef392e543352558b2263826e"; };
"@jimp/plugin-resize@npm:0.16.2" = { filename = "@jimp-plugin-resize-npm-0.16.2-8cf184075a-f8fdf9de6f.zip"; sha512 = "f8fdf9de6fd157c4d4f1eb4a80bd39bd32833fc05d5033ef438de52b30830746d3830d5d255f6a49fa59f25e210bc37c8a1f4301d7427b2ebd508c28c0fa4446"; };
"@jimp/plugin-rotate@npm:0.16.2" = { filename = "@jimp-plugin-rotate-npm-0.16.2-ccd5efe1f8-611ba1ca7f.zip"; sha512 = "611ba1ca7fdd905dabad836a2525c6c659e473f218b775e1940698ef97faf336f9a8b1818a4a54b3c569dda37fe3dc0b08faa80cc7dda06eb2990015069e7a08"; };
"@jimp/plugin-scale@npm:0.16.2" = { filename = "@jimp-plugin-scale-npm-0.16.2-aabffc4d92-3616cedb46.zip"; sha512 = "3616cedb46a129db65ca41e2511a0b071ae1ca25238c3e38d45a6e6739872b07135edc37537b191fcf5ab18aed5b943010181df79130b8b73ed5cd23a57c9383"; };
"@jimp/plugin-shadow@npm:0.16.2" = { filename = "@jimp-plugin-shadow-npm-0.16.2-d6bff0bf15-b29abee566.zip"; sha512 = "b29abee566b7b308e8fb98a05f9c68a098178e965267139d187b28dfb9b2c309eb336785dba20351515c5874a1b11540aec8438b49b4d52cfc9f3a47eaf72879"; };
"@jimp/plugin-threshold@npm:0.16.2" = { filename = "@jimp-plugin-threshold-npm-0.16.2-d577200c76-c3de882dfb.zip"; sha512 = "c3de882dfb63f9f08e906eb72c14981d43e12e7cc4436de37215b3e432e49e83717cd3ec2f00e6b18d73f17bc401bd7050fcb2f5b9803ed4772db4b1738b5928"; };
"@types/node@npm:16.9.1" = { filename = "@types-node-npm-16.9.1-bde6d3b0c9-41afcf183a.zip"; sha512 = "41afcf183a22d59323a0199dd7e0f46591247f45fc08a4434edb26d56dc279ae4fdb80f37989ddd7a0f45e3857c4933e6e82057ede09c5a829f77e373e680375"; };
"@jimp/plugin-color@npm:0.16.2" = { filename = "@jimp-plugin-color-npm-0.16.2-471fbd7f15-31d903afe9.zip"; sha512 = "31d903afe9eb979afe35fc854621879698f08fad3a7e9dc510960468cb10d31a0ed36fa2ac9938a00cf8f06eb331bdaaf2a393db6fce163c737a2f972a0b2c94"; };
"@jimp/plugin-print@npm:0.16.2" = { filename = "@jimp-plugin-print-npm-0.16.2-554179a07b-2a94720ef8.zip"; sha512 = "2a94720ef86e0cfb94393b8ff894034d0488b47166bab7c4c112cf1a06b8a583289cd9adc140a69188855251ece0d2e7e0be7b3844e0c7adcc428c432fe58cf0"; };
"psl@npm:1.9.0" = { filename = "psl-npm-1.9.0-a546edad1a-20c4277f64.zip"; sha512 = "20c4277f640c93d393130673f392618e9a8044c6c7bf61c53917a0fddb4952790f5f362c6c730a9c32b124813e173733f9895add8d26f566ed0ea0654b2e711d"; };
"punycode@npm:2.2.0" = { filename = "punycode-npm-2.2.0-9eb533dae1-32f291c1b1.zip"; sha512 = "32f291c1b1e8bef8a7d351a369579565bc17530ee5224d2f2b5c37b2647aa0ec7f1972294e2de1b632812f90c8080a7c0c5645c14758aadc0f27b35dd4906d89"; };
"universalify@npm:0.2.0" = { filename = "universalify-npm-0.2.0-9984e61c10-e86134cb12.zip"; sha512 = "e86134cb12919d177c2353196a4cc09981524ee87abf621f7bc8d249dbbbebaec5e7d1314b96061497981350df786e4c5128dbf442eba104d6e765bc260678b5"; };
"tr46@npm:2.1.0" = { filename = "tr46-npm-2.1.0-00af583f4f-ffe6049b9d.zip"; sha512 = "ffe6049b9dca3ae329b059aada7f515b0f0064c611b39b51ff6b53897e954650f6f63d9319c6c008d36ead477c7b55e5f64c9dc60588ddc91ff720d64eb710b3"; };
"infer-owner@npm:1.0.4" = { filename = "infer-owner-npm-1.0.4-685ac3d2af-181e732764.zip"; sha512 = "181e732764e4a0611576466b4b87dac338972b839920b2a8cde43642e4ed6bd54dc1fb0b40874728f2a2df9a1b097b8ff83b56d5f8f8e3927f837fdcb47d8a89"; };
"promise-inflight@npm:1.0.1" = { filename = "promise-inflight-npm-1.0.1-5bb925afac-2274948309.zip"; sha512 = "22749483091d2c594261517f4f80e05226d4d5ecc1fc917e1886929da56e22b5718b7f2a75f3807e7a7d471bc3be2907fe92e6e8f373ddf5c64bae35b5af3981"; };
"esprima@npm:4.0.1" = { filename = "esprima-npm-4.0.1-1084e98778-b45bc805a6.zip"; sha512 = "b45bc805a613dbea2835278c306b91aff6173c8d034223fa81498c77dcbce3b2931bf6006db816f62eacd9fd4ea975dfd85a5b7f3c6402cfd050d4ca3c13a628"; };
"estraverse@npm:5.3.0" = { filename = "estraverse-npm-5.3.0-03284f8f63-072780882d.zip"; sha512 = "072780882dc8416ad144f8fe199628d2b3e7bbc9989d9ed43795d2c90309a2047e6bc5979d7e2322a341163d22cfad9e21f4110597fe487519697389497e4e2b"; };
"esutils@npm:2.0.3" = { filename = "esutils-npm-2.0.3-f865beafd5-22b5b08f74.zip"; sha512 = "22b5b08f74737379a840b8ed2036a5fb35826c709ab000683b092d9054e5c2a82c27818f12604bfc2a9a76b90b6834ef081edbc1c7ae30d1627012e067c6ec87"; };
"source-map@npm:0.6.1" = { filename = "source-map-npm-0.6.1-1a3621db16-59ce8640cf.zip"; sha512 = "59ce8640cf3f3124f64ac289012c2b8bd377c238e316fb323ea22fbfe83da07d81e000071d7242cad7a23cd91c7de98e4df8830ec3f133cb6133a5f6e9f67bc2"; };
"object-assign@npm:4.1.1" = { filename = "object-assign-npm-4.1.1-1004ad6dec-fcc6e4ea8c.zip"; sha512 = "fcc6e4ea8c7fe48abfbb552578b1c53e0d194086e2e6bbbf59e0a536381a292f39943c6e9628af05b5528aa5e3318bb30d6b2e53cadaf5b8fe9e12c4b69af23f"; };
"p-map@npm:4.0.0" = { filename = "p-map-npm-4.0.0-4677ae07c7-cb0ab21ec0.zip"; sha512 = "cb0ab21ec0f32ddffd31dfc250e3afa61e103ef43d957cc45497afe37513634589316de4eb88abdfd969fe6410c22c0b93ab24328833b8eb1ccc087fc0442a1c"; };
"unique-filename@npm:1.1.1" = { filename = "unique-filename-npm-1.1.1-c885c5095b-cf4998c922.zip"; sha512 = "cf4998c9228cc7647ba7814e255dec51be43673903897b1786eff2ac2d670f54d4d733357eb08dea969aa5e6875d0e1bd391d668fbdb5a179744e7c7551a6f80"; };
"url-parse@npm:1.5.10" = { filename = "url-parse-npm-1.5.10-64fa2bcd6d-fbdba6b1d8.zip"; sha512 = "fbdba6b1d83336aca2216bbdc38ba658d9cfb8fc7f665eb8b17852de638ff7d1a162c198a8e4ed66001ddbf6c9888d41e4798912c62b4fd777a31657989f7bdf"; };
"buffer-equal@npm:0.0.1" = { filename = "buffer-equal-npm-0.0.1-428f7d217a-ca4b52e6c0.zip"; sha512 = "ca4b52e6c01143529d957a78cb9a93e4257f172bbab30d9eb87c20ae085ed23c5e07f236ac051202dacbf3d17aba42e1455f84cba21ea79b67d57f2b05e9a613"; };
"mime@npm:1.6.0" = { filename = "mime-npm-1.6.0-60ae95038a-fef25e3926.zip"; sha512 = "fef25e39263e6d207580bdc629f8872a3f9772c923c7f8c7e793175cee22777bbe8bba95e5d509a40aaa292d8974514ce634ae35769faa45f22d17edda5e8557"; };
"parse-bmfont-ascii@npm:1.0.6" = { filename = "parse-bmfont-ascii-npm-1.0.6-15dd9909ba-de3f6671f1.zip"; sha512 = "de3f6671f183c3e9d64bb4812b0407693b5fd0d24e9d16b2e106bb9eef809d64a6cc061f39ca29bb10c5c2e47e241e91b7aeefa587391fff7ccb27ab9db5012e"; };
"parse-bmfont-binary@npm:1.0.6" = { filename = "parse-bmfont-binary-npm-1.0.6-50892a2477-ca37fb1e92.zip"; sha512 = "ca37fb1e92f5941fddc5342b45857fafd27f00d2bd5fa44dd504bec6faeab97536c95ad45260c2dd5fc4c63de71e525663d3cdac09d038cbca803d97c669add5"; };
"xtend@npm:4.0.2" = { filename = "xtend-npm-4.0.2-7f2375736e-ac5dfa738b.zip"; sha512 = "ac5dfa738b21f6e7f0dd6e65e1b3155036d68104e67e5d5d1bde74892e327d7e5636a076f625599dc394330a731861e87343ff184b0047fef1360a7ec0a5a36a"; };
"@npmcli/fs@npm:1.1.1" = { filename = "@npmcli-fs-npm-1.1.1-17f582e0b6-f5ad92f157.zip"; sha512 = "f5ad92f157ed222e4e31c352333d0901df02c7c04311e42a81d8eb555d4ec4276ea9c635011757de20cc476755af33e91622838de573b17e52e2e7703f0a9965"; };
"@npmcli/move-file@npm:1.1.2" = { filename = "@npmcli-move-file-npm-1.1.2-4f6c7b3354-c96381d4a3.zip"; sha512 = "c96381d4a37448ea280951e46233f7e541058cf57a57d4094dd4bdcaae43fa5872b5f2eb6bfb004591a68e29c5877abe3cdc210cb3588cbf20ab2877f31a7de7"; };
"parse-bmfont-xml@npm:1.1.4" = { filename = "parse-bmfont-xml-npm-1.1.4-dcbad8a521-879e5435be.zip"; sha512 = "879e5435be44f22b8c4934e2e1d2754a6d90a9ddb16309360daff965e1428d877b673f3d1fafaab4fef437c912a0db9f85545e0dd375ec62df7d4d328450d257"; };
"querystringify@npm:2.2.0" = { filename = "querystringify-npm-2.2.0-4e77c9f606-5641ea231b.zip"; sha512 = "5641ea231bad7ef6d64d9998faca95611ed4b11c2591a8cae741e178a974f6a8e0ebde008475259abe1621cb15e692404e6b6626e927f7b849d5c09392604b15"; };
"requires-port@npm:1.0.0" = { filename = "requires-port-npm-1.0.0-fd036b488a-eee0e303ad.zip"; sha512 = "eee0e303adffb69be55d1a214e415cf42b7441ae858c76dfc5353148644f6fd6e698926fc4643f510d5c126d12a705e7c8ed7e38061113bdf37547ab356797ff"; };
"xhr@npm:2.6.0" = { filename = "xhr-npm-2.6.0-c4a4d64db8-a1db277e37.zip"; sha512 = "a1db277e37737caf3ed363d2a33ce4b4ea5b5fc190b663a6f70bc252799185b840ccaa166eaeeea4841c9c60b87741f0a24e29cbcf6708dd425986d4df186d2f"; };
"@gar/promisify@npm:1.1.2" = { filename = "@gar-promisify-npm-1.1.2-2343f94380-d05081e088.zip"; sha512 = "d05081e0887a49c178b75ee3067bd6ee086f73c154d121b854fb2e044e8a89cb1cbb6de3a0dd93a519b80f0531fda68b099dd7256205f7fbb3490324342f2217"; };
"unique-slug@npm:2.0.2" = { filename = "unique-slug-npm-2.0.2-f6ba1ddeb7-5b6876a645.zip"; sha512 = "5b6876a645da08d505dedb970d1571f6cebdf87044cb6b740c8dbb24f0d6e1dc8bdbf46825fd09f994d7cf50760e6f6e063cfa197d51c5902c00a861702eb75a"; };
"optionator@npm:0.8.3" = { filename = "optionator-npm-0.8.3-bc555bc5b7-b8695ddf3d.zip"; sha512 = "b8695ddf3d593203e25ab0900e265d860038486c943ff8b774f596a310f8ceebdb30c6832407a8198ba3ec9debe1abe1f51d4aad94843612db3b76d690c61d34"; };
"aggregate-error@npm:3.1.0" = { filename = "aggregate-error-npm-3.1.0-415a406f4e-1101a33f21.zip"; sha512 = "1101a33f21baa27a2fa8e04b698271e64616b886795fd43c31068c07533c7b3facfcaf4e9e0cab3624bd88f729a592f1c901a1a229c9e490eafce411a8644b79"; };
"xml-parse-from-string@npm:1.0.1" = { filename = "xml-parse-from-string-npm-1.0.1-8305dfb2e2-5155cb98e4.zip"; sha512 = "5155cb98e428409829f4060ce542c55438b2f7646d11fd306d850eaf12d35c06ffd9e86d76aa5230121a533b958fd1a319d6f90a5c113391853d0ff01f4da7bb"; };
"xml2js@npm:0.4.23" = { filename = "xml2js-npm-0.4.23-93a8b2e10b-ca0cf2dfbf.zip"; sha512 = "ca0cf2dfbf6deeaae878a891c8fbc0db6fd04398087084edf143cdc83d0509ad0fe199b890f62f39c4415cf60268a27a6aed0d343f0658f8779bd7add690fa98"; };
"is-function@npm:1.0.2" = { filename = "is-function-npm-1.0.2-90ac01927b-7d564562e0.zip"; sha512 = "7d564562e07b4b51359547d3ccc10fb93bb392fd1b8177ae2601ee4982a0ece86d952323fc172a9000743a3971f09689495ab78a1d49a9b14fc97a7e28521dc0"; };
"parse-headers@npm:2.0.5" = { filename = "parse-headers-npm-2.0.5-c1f2d79179-3e97f01e4c.zip"; sha512 = "3e97f01e4c7f960bfbfd0ee489f0bd8d3c72b6c814f1f79b66abec2cca8eaf8e4ecd89deba0b6e61266469aed87350bc932001181c01ff8c29a59e696abe251f"; };
"deep-is@npm:0.1.4" = { filename = "deep-is-npm-0.1.4-88938b5a67-edb65dd0d7.zip"; sha512 = "edb65dd0d7d1b9c40b2f50219aef30e116cedd6fc79290e740972c132c09106d2e80aa0bc8826673dd5a00222d4179c84b36a790eef63a4c4bca75a37ef90804"; };
"fast-levenshtein@npm:2.0.6" = { filename = "fast-levenshtein-npm-2.0.6-fcd74b8df5-92cfec0a8d.zip"; sha512 = "92cfec0a8dfafd9c7a15fba8f2cc29cd0b62b85f056d99ce448bbcd9f708e18ab2764bda4dd5158364f4145a7c72788538994f0d1787b956ef0d1062b0f7c24c"; };
"prelude-ls@npm:1.1.2" = { filename = "prelude-ls-npm-1.1.2-a0daac0886-c4867c8748.zip"; sha512 = "c4867c87488e4a0c233e158e4d0d5565b609b105d75e4c05dc760840475f06b731332eb93cc8c9cecb840aa8ec323ca3c9a56ad7820ad2e63f0261dadcb154e4"; };
"word-wrap@npm:1.2.3" = { filename = "word-wrap-npm-1.2.3-7fb15ab002-30b48f91fc.zip"; sha512 = "30b48f91fcf12106ed3186ae4fa86a6a1842416df425be7b60485de14bec665a54a68e4b5156647dec3a70f25e84d270ca8bc8cd23182ed095f5c7206a938c1f"; };
"clean-stack@npm:2.2.0" = { filename = "clean-stack-npm-2.2.0-a8ce435a5c-2ac8cd2b2f.zip"; sha512 = "2ac8cd2b2f5ec986a3c743935ec85b07bc174d5421a5efc8017e1f146a1cf5f781ae962618f416352103b32c9cd7e203276e8c28241bbe946160cab16149fb68"; };
"indent-string@npm:4.0.0" = { filename = "indent-string-npm-4.0.0-7b717435b2-824cfb9929.zip"; sha512 = "824cfb9929d031dabf059bebfe08cf3137365e112019086ed3dcff6a0a7b698cb80cf67ccccde0e25b9e2d7527aa6cc1fed1ac490c752162496caba3e6699612"; };
"type-check@npm:0.3.2" = { filename = "type-check-npm-0.3.2-a4a38bb0b6-dd3b149564.zip"; sha512 = "dd3b1495642731bc0e1fc40abe5e977e0263005551ac83342ecb6f4f89551d106b368ec32ad3fb2da19b3bd7b2d1f64330da2ea9176d8ddbfe389fb286eb5124"; };
"global@npm:4.4.0" = { filename = "global-npm-4.4.0-888ee8033d-9c057557c8.zip"; sha512 = "9c057557c8f5a5bcfbeb9378ba4fe2255d04679452be504608dd5f13b54edf79f7be1db1031ea06a4ec6edd3b9f5f17d2d172fb47e6c69dae57fd84b7e72b77f"; };
"levn@npm:0.3.0" = { filename = "levn-npm-0.3.0-48d774b1c2-0d084a5242.zip"; sha512 = "0d084a524231a8246bb10fec48cdbb35282099f6954838604f3c7fc66f2e16fa66fd9cc2f3f20a541a113c4dafdf181e822c887c8a319c9195444e6c64ac395e"; };
"xmlbuilder@npm:11.0.1" = { filename = "xmlbuilder-npm-11.0.1-b8b04dc929-7152695e16.zip"; sha512 = "7152695e16f1a9976658215abab27e55d08b1b97bca901d58b048d2b6e106b5af31efccbdecf9b07af37c8377d8e7e821b494af10b3a68b0ff4ae60331b415b0"; };
"process@npm:0.11.10" = { filename = "process-npm-0.11.10-aeb3b641ae-bfcce49814.zip"; sha512 = "bfcce49814f7d172a6e6a14d5fa3ac92cc3d0c3b9feb1279774708a719e19acd673995226351a082a9ae99978254e320ccda4240ddc474ba31a76c79491ca7c3"; };
"min-document@npm:2.19.0" = { filename = "min-document-npm-2.19.0-458cdb3d84-da6437562e.zip"; sha512 = "da6437562ea2228041542a2384528e74e22d1daa1a4ec439c165abf0b9d8a63e17e3b8a6dc6e0c731845e85301198730426932a0e813d23f932ca668340c9623"; };
"dom-walk@npm:0.1.2" = { filename = "dom-walk-npm-0.1.2-7d20a1a8d8-19eb0ce9c6.zip"; sha512 = "19eb0ce9c6de39d5e231530685248545d9cd2bd97b2cb3486e0bfc0f2a393a9addddfd5557463a932b52fdfcf68ad2a619020cd2c74a5fe46fbecaa8e80872f3"; };
};

in optionalOverride overrideAttrs project
